#include "stdafx.h"

#include "wb_config.h"

#include "table_figure.h"
#include "connection_figure.h"
/**
 * @file  table_figure.cpp
 * @brief 
 */

using namespace wbfig;





TableColumnItem::TableColumnItem(mdc::Layer *layer)
: FigureItem(layer)
{
}



void TableColumnItem::draw_state(mdc::CairoCtx *cr)
{
  switch (get_state())
  {
  case Hovering:
    cr->save();
    cr->set_color(get_view()->get_hover_color(), 0.5);
    stroke_outline(cr, 0);
    cr->fill();
    cr->restore();
    break;
  case Highlighted:
    cr->save();
    cr->set_color(get_view()->get_hover_color(), 0.3);
    stroke_outline(cr, 0);
    cr->fill();
    cr->restore();
    break;
  default:
    mdc::IconTextFigure::draw_state(cr);
  }
}

//-------------------------------------------------------------------------------------------

Table::Table(mdc::Layer *layer, bool collapsible)
: BaseFigure(layer), _background(layer), _title(layer, collapsible)
{
  _original_column_box_height= 0.0;

  _hide_columns= false;
  _hide_indexes= false;
  _hide_triggers= false;

  add_magnet(_sides_magnet= new mdc::BoxSideMagnet(this));
  _sides_magnet->set_compare_slot(sigc::mem_fun(this, &Table::compare_connection_position));
}


bool Table::on_enter(mdc::CanvasItem *target, const mdc::Point &point)
{
  _signal_enter.emit();
  return super::on_enter(target, point);
}


bool Table::on_leave(mdc::CanvasItem *target, const mdc::Point &point)
{
  _signal_leave.emit();
  return super::on_enter(target, point);
}


void Table::set_title_font(const mdc::FontSpec &font)
{
  _title.set_font(font);
}


void Table::set_section_font(const mdc::FontSpec &font)
{
  if (get_index_title())
    get_index_title()->set_font(font);
  if (get_trigger_title())
    get_trigger_title()->set_font(font);
}


void Table::set_content_font(const mdc::FontSpec &font)
{
  super::set_content_font(font);
}


wbfig::FigureItem *Table::create_table_item(mdc::Layer *layer)
{
  return new TableColumnItem(layer);
}


bool Table::compare_connection_position(mdc::Connector *a, mdc::Connector *b, 
                                        mdc::BoxSideMagnet::Side side)
{
  wbfig::ConnectionLineLayouter *layouter;
  mdc::Point a_pos, b_pos;
  mdc::CanvasItem *item;
  mdc::Connector *a_conn, *b_conn;

  layouter= dynamic_cast<wbfig::ConnectionLineLayouter*>(
    dynamic_cast<mdc::Line*>(a->get_owner())->get_layouter());
  if (!layouter) return false;
  if (layouter->get_start_connector() == a)
  {
    a_conn= layouter->get_end_connector();
    item= a_conn->get_connected_item();
  }
  else
  {
    a_conn= layouter->get_start_connector();
    item= a_conn->get_connected_item();
  }
  if (item)
    a_pos= item->get_root_bounds().center();
  else
    a_pos= a_conn->get_position();

  layouter= dynamic_cast<wbfig::ConnectionLineLayouter*>(
    dynamic_cast<mdc::Line*>(b->get_owner())->get_layouter());
  if (!layouter) return false;
  if (layouter->get_start_connector() == b)
  {
    b_conn= layouter->get_end_connector();
    item= b_conn->get_connected_item();
  }
  else
  {
    b_conn= layouter->get_start_connector();
    item= b_conn->get_connected_item();
  }
  if (item)
    b_pos= item->get_root_bounds().center();
  else
    b_pos= b_conn->get_position();

  // the strict ordering between 2 connectors is important 
  // ie, compare(a,b) and compare(b,a) must never be the same
  // otherwise we get an infinite loop when a_pos == b_pos
  if (side == mdc::BoxSideMagnet::Top || side == mdc::BoxSideMagnet::Bottom)
    return a_pos.x < b_pos.x || a_pos.x == b_pos.x && a_conn < b_conn;
  else
    return a_pos.y < b_pos.y || a_pos.y == b_pos.y && a_conn < b_conn;
}


