#ifndef __GRT_SHELL_H__
#define __GRT_SHELL_H__

namespace MySQL {
  namespace Grt {

    public ref class GrtShell 
    {
    public:
      delegate void VoidStringDelegate(String^ str);

    private:
      delegate void VoidStringWrapperDelegate(const std::string& str);
      typedef void (*GrtShell::VOID_STRING_HANDLER_CB)(const std::string& str);

      ::bec::ShellBE *inner;

      VoidStringDelegate^ ready_handler_delegate;
      VoidStringWrapperDelegate^ ready_handler_wrapper_delegate;

      void ready_handler_wrapper(const std::string& str)
      {
        ready_handler_delegate(GrtUtil::CharToString(str.c_str()));
      }

      VoidStringDelegate^ output_handler_delegate;
      VoidStringWrapperDelegate^ output_handler_wrapper_delegate;

      void output_handler_wrapper(const std::string& str)
      {
        output_handler_delegate(GrtUtil::CharToString(str.c_str()));
      }

    public:
      //GrtShell(GRT^ grt, GRTDispatcher *dispatcher);

      GrtShell(::bec::ShellBE *inn)
        : inner(inn)
      {}

      ~GrtShell()
      {}

      void enable_completion()
      { inner->enable_completion(); }

      void set_save_directory(String^ path)
      {
        char *c_str = GrtUtil::StringToChar(path);
        inner->set_save_directory(std::string(c_str));
        delete c_str;
      }

      void start()
      { inner->start(); }

      void process_line_async(String^ line)
      {
        char *c_str = GrtUtil::StringToChar(line);
        inner->process_line_async(std::string(c_str));
        delete c_str;
      }

      bool previous_history_line(String^ current_line, [Out] String^ %line)
      {
        std::string s1;

        char *c_str= GrtUtil::StringToChar(current_line);
        bool retval= inner->previous_history_line(std::string(c_str), s1);
        delete c_str;

        line= GrtUtil::CharToString(s1.c_str());

        return retval;
      }

      bool next_history_line([Out] String^ %line)
      {
        std::string s1;

        bool retval= inner->next_history_line(s1);

        line= GrtUtil::CharToString(s1.c_str());

        return retval;
      }

      void reset_history_position()
      {
        inner->reset_history_position();
      }

      void write_line(String^ line)
      {
        char *c_str = GrtUtil::StringToChar(line);
        inner->write_line(std::string(c_str));
        delete c_str;
      }

      void write(String^ text)
      {
        char *c_str = GrtUtil::StringToChar(text);
        inner->write_line(std::string(c_str));
        delete c_str;
      }

      //void writef(const char *fmt, ...);

      void set_ready_handler(VoidStringDelegate^ dt)
      {
        ready_handler_delegate= dt;
        ready_handler_wrapper_delegate = gcnew VoidStringWrapperDelegate(this, &GrtShell::ready_handler_wrapper);
        IntPtr ip = Marshal::GetFunctionPointerForDelegate(ready_handler_wrapper_delegate);
        VOID_STRING_HANDLER_CB cb = static_cast<VOID_STRING_HANDLER_CB>(ip.ToPointer());
        inner->set_ready_handler(sigc::ptr_fun(cb));
      }

      void set_saves_history(bool flag)
      { inner->set_saves_history(flag); }

      String^ get_snippet_data() 
      {
        return GrtUtil::CharToString(inner->get_snippet_data().c_str()); 
      }

      void set_snippet_data(String^ data) 
      {
        char *c_str = GrtUtil::StringToChar(data);
        inner->set_snippet_data(c_str); 
        delete c_str;
      }

      void store_history() { inner->store_history(); }

      void restore_history() { inner->restore_history(); }

      //GList *complete_word(const std::string &prefix, std::string &nprefix);
    };

  } // namespace Grt
} // namespace MySQL

#endif // __GRT_SHELL_H__