/**
 * @file  wb_context_physical.cpp
 * @brief 
 */


#include "stdafx.h"

#include "wb_context_physical.h"
#include "wb_find_dialog.h"

#include "grts/structs.workbench.physical.h"

#include "grtdb/db_object_helpers.h"


#include "wbcanvas/workbench_physical_model.h"
#include "wbcanvas/workbench_physical_view.h"
#include "wbcanvas/workbench_physical_tablefigure.h"
#include "wbcanvas/workbench_physical_viewfigure.h"
#include "wbcanvas/workbench_physical_routinegroupfigure.h"
#include "wbcanvas/workbench_physical_connection.h"

#define FILE_CONNECTION_LIST "connections.xml"

using namespace grt;
using namespace bec;
using namespace wb;



WBContextPhysical::WBContextPhysical(bool verbose)
: WBContextBase(verbose)
{
  _schemata_listener= 0;
  _catalog_tree= 0;
}


WBContextPhysical::~WBContextPhysical()
{
  delete _catalog_tree;
}


void WBContextPhysical::load_app_options(bool update)
{
  if (!update)
  {
    app_ToolbarRef tools_toolbar(app_ToolbarRef::cast_from(
      _manager->get_grt()->unserialize("data/tools_toolbar_physical.xml")));

    _toolbars[tools_toolbar.name()]= tools_toolbar;

    app_ToolbarRef toolbar;
    toolbar= app_ToolbarRef::cast_from(
      _manager->get_grt()->unserialize("data/main_toolbar_physical_table.xml"));
    _toolbars[toolbar.name()]= toolbar;

    toolbar= app_ToolbarRef::cast_from(
      _manager->get_grt()->unserialize("data/main_toolbar_physical_view.xml"));
    _toolbars[toolbar.name()]= toolbar;

    toolbar= app_ToolbarRef::cast_from(
      _manager->get_grt()->unserialize("data/main_toolbar_physical_routinegroup.xml"));
    _toolbars[toolbar.name()]= toolbar;

    toolbar= app_ToolbarRef::cast_from(
      _manager->get_grt()->unserialize("data/main_toolbar_physical_relationship.xml"));
    _toolbars["main/"WB_TOOL_PREL11_NOID]= toolbar;
    _toolbars["main/"WB_TOOL_PREL1n_NOID]= toolbar;
    _toolbars["main/"WB_TOOL_PREL11]= toolbar;
    _toolbars["main/"WB_TOOL_PREL1n]= toolbar;
    _toolbars["main/"WB_TOOL_PRELnm]= toolbar;
  }

  db_mgmt_ManagementRef mgmt= get_root().rdbmsMgmt();
  std::string conn_list_xml= make_path(_user_datadir, FILE_CONNECTION_LIST);
  if (g_file_test(conn_list_xml.c_str(), G_FILE_TEST_EXISTS))
  {
    try
    {
      grt::ListRef<db_mgmt_Connection> 
        list(grt::ListRef<db_mgmt_Connection>::cast_from(_manager->get_grt()->unserialize(conn_list_xml)));

      if (list.is_valid())
      {
        mgmt.storedConns().remove_all();
        for (size_t c= list.count(), i= 0; i < c; i++)
          mgmt.storedConns().insert(list.get(i));
      }
    }
    catch (std::exception &exc)
    {
      _manager->get_grt()->send_warning(fmt("Error loading '%s': %s", conn_list_xml.c_str(), exc.what()));
    }
  }
}


void WBContextPhysical::save_app_options()
{
  _manager->get_grt()->serialize(get_root().rdbmsMgmt().storedConns(),
    make_path(_user_datadir, FILE_CONNECTION_LIST));
}


/*
void WBContextPhysical::finder(FindDialogBE *finder, const model_ModelRef &model)
{
  if (model.is_instance(workbench_physical_Model::static_class_name()))
  {
    workbench_physical_ModelRef pmodel(workbench_physical_ModelRef::cast_from(model));

    for (size_t c= model.views().count(), i= 0; i < c; i++)
    {
      model_ViewRef view(model.views().get(i));

      for (size_t d= view.figures().count(), j= 0; j < d; j++)
      {
        model_FigureRef figure(view.figures().get(j));

        if (figure.is_instance(workbench_physical_TableFigure::static_class_name()))
        {
          workbench_physical_TableFigureRef table(workbench_physical_TableFigureRef::cast_from(figure));

          if (finder->match(table.table().name()))
            finder->add_match(table, table.table(), table.table().name());

          if (finder->get_search_in_comments() && finder->match(table.table().comment()))
            finder->add_match(table, table.table(), table.table().comment());

          grt::ListRef<db_Column> columns(table.table().columns());
          for (size_t cc= columns.count(), ci= 0; ci < cc; ci++)
          {
            if (finder->match(columns.get(ci).name()))
              finder->add_match(table, columns.get(ci), columns.get(ci).name());

            if (finder->get_search_in_comments() && finder->match(columns.get(ci).comment()))
              finder->add_match(table, columns.get(ci), columns.get(ci).comment());
          }
        }
        else if (figure.is_instance(workbench_physical_ViewFigure::static_class_name()))
        {
          workbench_physical_ViewFigureRef viewf(workbench_physical_ViewFigureRef::cast_from(figure));

          if (finder->match(viewf.view().name()))
            finder->add_match(viewf, viewf.view(), viewf.view().name());

          if (finder->get_search_in_comments() && finder->match(viewf.view().comment()))
            finder->add_match(viewf, viewf.view(), viewf.view().comment());
        }
        else if (figure.is_instance(workbench_physical_RoutineGroupFigure::static_class_name()))
        {
          workbench_physical_RoutineGroupFigureRef rg(workbench_physical_RoutineGroupFigureRef::cast_from(figure));

          if (finder->match(rg.routineGroup().name()))
            finder->add_match(rg, rg.routineGroup(), rg.routineGroup().name());

          if (finder->get_search_in_comments() && finder->match(rg.routineGroup().comment()))
            finder->add_match(rg, rg.routineGroup(), rg.routineGroup().comment());

          grt::ListRef<db_Routine> routines(rg.routineGroup().routines());

          for (size_t rc= routines.count(), r= 0; r < rc; r++)
          {
            if (finder->match(routines.get(r).name()))
              finder->add_match(rg, routines.get(r), routines.get(r).name());

            if (finder->get_search_in_comments() && finder->match(routines.get(r).comment()))
              finder->add_match(rg, routines.get(r), routines.get(r).comment());
          }
        }
      }
    }
  }
}
*/


void WBContextPhysical::setup_context_grt(grt::GRT *grt, WBOptions *options)
{
  //XXX must be loaded from DB module
  db_mgmt_RdbmsRef rdbms= db_mgmt_RdbmsRef::cast_from(grt->unserialize(make_path(options->basedir, "modules/data/mysql_rdbms_info.xml")));
  
  get_root().rdbmsMgmt().rdbms().insert(rdbms);

//  _find_dialog->add_custom_search_function(sigc::mem_fun(this, &WBContextPhysical::finder));
}



void WBContextPhysical::init_catalog_grt(const db_mgmt_RdbmsRef &rdbms,
                                 const std::string &db_versionRef,
                                 workbench_physical_ModelRef &model)
{
  grt::GRT *grt= _manager->get_grt();
  
  std::string db_package= rdbms.databaseObjectPackage();

  // assemble struct name for catalog and schema for the requested db type

  std::string catalog_struct= db_package+".Catalog";
  std::string schema_struct= db_package+".Schema";
  
  if (!grt->get_struct(catalog_struct).is_valid() ||
      !grt->get_struct(schema_struct).is_valid())
  {
    // Struct definition for '%s' and/or '%s' cannot be found
    throw grt_runtime_error("Support for RDBMS "+db_package+" not found.",
                            "Struct definition for "+catalog_struct+" and/or "+schema_struct+" could not be found");
  }

  db_CatalogRef catalog(grt->create_object<db_Catalog>(catalog_struct.c_str(),
                                                    "name", StringRef("default").valueptr(),
                                                    "owner", model.valueptr(),
                                                    NULL));
  //catalog.oldName(catalog.name());

  // set version
  GrtVersion version(grt);
  version.name("Version");
  version.owner(catalog);

  char **toks= g_strsplit(db_version.c_str(), ".", 0);

  if (toks[0])
    version.majorNumber(atoi(toks[0]));
  if (toks[1])
    version.minorNumber(atoi(toks[1]));
  if (toks[2])
    version.releaseNumber(atoi(toks[2]));

  g_strfreev(toks);


  catalog.simpleDatatypes(rdbms.simpleDatatypes());
  catalog.characterSets(rdbms.characterSets());

  model.catalog(catalog);

  // add listener for any operation on the schema list
  reset_schemata_listeners();

  // global listeners for changes in db objects (table renames etc)
  grt->add_listener(db_Table::static_class_name(), myx_notify_object_changed,
    sigc::mem_fun(this, &WBContextPhysical::db_object_changed));
  grt->add_listener(db_View::static_class_name(), myx_notify_object_changed,
    sigc::mem_fun(this, &WBContextPhysical::db_object_changed));
  grt->add_listener(db_RoutineGroup::static_class_name(), myx_notify_object_changed,
    sigc::mem_fun(this, &WBContextPhysical::db_object_changed));

#if 1
  // create an initial schema
  
  db_SchemaRef schema(grt->create_object<db_Schema>(schema_struct.c_str(),
                                                 "name", StringRef("mydb").valueptr(),
                                                 "oldName", StringRef("mydb").valueptr(),
                                                 "owner", catalog.valueptr(),
                                                 "defaultCharacterSetName", StringRef("latin1").valueptr(),
                                                 NULL));
  //schema.oldName(schema.name());

  catalog.schemata().insert(schema);
#endif
}




void WBContextPhysical::setup_physical_model(grt::GRT *grt, workbench_DocumentRef &doc, 
                                     const std::string &rdbms_name, const std::string &rdbms_version)
{
  // init physical model
  workbench_physical_ModelRef pmodel(grt);
  pmodel.owner(doc);

  pmodel.connectionNotation(get_wb_options().get_string("DefaultConnectionNotation"));
  pmodel.figureNotation(get_wb_options().get_string("DefaultFigureNotation"));

  doc.physicalModels().insert(pmodel);

  db_mgmt_ManagementRef mgmt= db_mgmt_ManagementRef::cast_from(grt->get("/wb/rdbmsMgmt"));
  
  // find the rdbms module for the db we want
  db_mgmt_RdbmsRef rdbms;

  rdbms= grt::find_named_object_in_list<db_mgmt_Rdbms>(mgmt.rdbms(), rdbms_name);
  if (!rdbms.is_valid())
  {
    throw grt_runtime_error("Could not locate RDBMS support object for "+rdbms_name,
                            "new_physical(): There is no RDBMS object with the name "+rdbms_name+" in the /rdbmsMgmt/rdbms list.");
  }
  pmodel.rdbms(rdbms);

  init_catalog_grt(rdbms, rdbms_version, pmodel);

  init_properties_grt(doc);
}


void WBContextPhysical::init_canvas_grt(grt::GRT *grt, WBOptions *options)
{
  ObjectBridgeBase::register_bridge<wbbridge::physical::Model>(grt, this);
  ObjectBridgeBase::register_bridge<wbbridge::physical::View>(grt);

  ObjectBridgeBase::register_bridge<wbbridge::physical::TableFigure>(grt);
  ObjectBridgeBase::register_bridge<wbbridge::physical::ViewFigure>(grt);
  ObjectBridgeBase::register_bridge<wbbridge::physical::RoutineGroupFigure>(grt);
  ObjectBridgeBase::register_bridge<wbbridge::physical::Connection>(grt);
}


//--------------------------------------------------------------------------------
// Document


//--------------------------------------------------------------------------------
// Model Management

grt::ValueRef WBContextPhysical::add_new_db_schema_grt(grt::GRT *grt,
                                                const std::string &model_id)
{
  workbench_physical_ModelRef model;
  db_SchemaRef schema;
  std::string name;
  std::string class_name;

  model= grt::find_object_in_list(get_document().physicalModels(), model_id);
  if (!model.is_valid())
    throw grt::grt_runtime_error("Cannot create new schema, target model not found",
                             "The model '"+model_id+"' was not found in the document.");

  class_name= *model.rdbms().databaseObjectPackage()+".Schema";

  name= grt::get_name_suggestion_for_list_object(
    grt::ObjectListRef::cast_from(model.catalog().schemata()), "new_schema");

  schema= grt->create_object<db_Schema>(class_name.c_str(),
                                        "owner", model.catalog().valueptr(),
                                        "name", grt::StringRef(name).valueptr(),
                                        NULL);

  _undo_manager->add_undo(new UndoListAddAction(model.catalog().schemata()));
  _undo_manager->set_action_description(_("Create New Schema"));

  model.catalog().schemata().insert(schema);

  return schema;
}


void WBContextPhysical::add_new_db_schema(const std::string &model_id)
{
  grt::ValueRef res;

  res= execute_in_grt_thread("Create new schema",
                        sigc::bind<std::string>(sigc::mem_fun(this, &WBContextPhysical::add_new_db_schema_grt),
                                                            model_id));

  if (res.is_valid())
  {
    show_status_text(fmt(_("Schema '%s' created."), db_SchemaRef::cast_from(res).name().c_str()));

    open_object_editor(GrtObjectRef::cast_from(res));
  }
  else
    show_status_text(_("Could not create new schema."));
}


grt::ValueRef WBContextPhysical::delete_db_schema_grt(grt::GRT *grt, const std::string &schema_id, 
                                                          bool check_empty)
{
  db_SchemaRef schema= db_SchemaRef::cast_from(grt->find_object(schema_id, "/wb/doc/physicalModels"));

  if (!schema.is_valid())
    throw grt::grt_runtime_error(_("Could not find schema to be removed."),
                             "Schema "+schema_id+" not found.");

  if (check_empty && 
      (schema.tables().count() > 0 || schema.views().count() > 0 || schema.routines().count() > 0))
  {
    return grt::DictRef::create(grt, "name", schema.name().valueptr(),
                                  "tables", grt::IntegerRef((int)schema.tables().count()).valueptr(),
                                  "views", grt::IntegerRef((int)schema.views().count()).valueptr(),
                                  "routines", grt::IntegerRef((int)schema.routines().count()).valueptr(),
                                  NULL);
  }

  workbench_physical_ModelRef model(get_parent_for_object<workbench_physical_Model>(schema));
  if (model.is_valid())
  {
    model_ViewRef view;

    _undo_manager->begin_undo_group();

    for (size_t vc= model.views().count(), vi= 0; vi < vc; vi++)
    {
      view= model.views().get(vi);

      // remove canvas objects for schema contents
      for (size_t c= schema.tables().count(), i= 0; i < c; i++)
      {
        std::string id= schema.tables().get(i).id();
        std::multimap<std::string,model_Figure>::iterator end, iter= _dbobject_figures.begin();
        end= _dbobject_figures.end();

        while (iter != end && iter->first == id)
        {
          delete_physical_model_object(iter->second, true);
          ++iter;
        }
      }

      for (size_t c= schema.views().count(), i= 0; i < c; i++)
      {
        std::string id= schema.views().get(i).id();
        std::multimap<std::string,model_Figure>::iterator end, iter= _dbobject_figures.begin();
        end= _dbobject_figures.end();

        while (iter != end && iter->first == id)
        {
          delete_physical_model_object(iter->second, true);
          ++iter;
        }
      }

      for (size_t c= schema.routineGroups().count(), i= 0; i < c; i++)
      {
        std::string id= schema.routineGroups().get(i).id();
        std::multimap<std::string,model_Figure>::iterator end, iter= _dbobject_figures.begin();
        end= _dbobject_figures.end();

        while (iter != end && iter->first == id)
        {
          delete_physical_model_object(iter->second, true);
          ++iter;
        }
      }
    }
    // remove schema
    _undo_manager->add_undo(new UndoListRemoveAction(model.catalog().schemata(), schema));

    model.catalog().schemata().remove_value(schema);

    _undo_manager->end_undo_group();
    _undo_manager->set_action_description(_("Delete Schema"));
  }

  return grt::ValueRef();
}


void WBContextPhysical::delete_db_schema(const std::string &schema_id)
{
  grt::ValueRef result;
  
  show_status_text(_("Deleting schema..."));

  result= execute_in_grt_thread("Delete schema",
            sigc::bind<std::string, bool>(sigc::mem_fun(this, &WBContextPhysical::delete_db_schema_grt),
                                          schema_id, true));

  if (result.is_valid() && result.type() == DictType)
  {
    grt::DictRef info(grt::DictRef::cast_from(result));
    int res;
    
    res= confirm_action(_("Delete Schema"),
                        fmt(_("The schema '%s' contains objects (%i tables, %i views, %i routines)."
                              "Do you want to delete it with all its contents?"),
                            info.get_string("name").c_str(),
                            *info.get_int("tables"), *info.get_int("views"), *info.get_int("routines")),
                        _("Delete"), _("Cancel"), "");
    if (res != 1)
    {
      show_status_text(_("Delete schema cancelled."));
      return;
    }

    result= execute_in_grt_thread("Delete schema",
             sigc::bind<std::string, bool>(sigc::mem_fun(this, &WBContextPhysical::delete_db_schema_grt),
                                           schema_id, false));
  }

  if (!result.is_valid())
    show_status_text(_("Schema deleted."));
  else
    show_status_text(_("Could not delete schema."));
}



grt::ValueRef WBContextPhysical::add_new_db_table_grt(grt::GRT *grt,
                                                  const std::string &schema_id)
{
  db_SchemaRef schema;
  db_TableRef table;
  std::string class_name;

  schema= db_SchemaRef::cast_from(grt->find_object(schema_id, "/wb/doc/physicalModels"));
  if (!schema.is_valid())
    throw grt::grt_runtime_error("Cannot create table, target schema not found", 
                             "Schema '"+schema_id+"' was not found");

  class_name= *get_parent_for_object<workbench_physical_Model>(schema).rdbms().databaseObjectPackage()+".Table";

  std::string name= grt::get_name_suggestion_for_list_object(grt::ObjectListRef::cast_from(schema.tables()), "table");

  table= grt->create_object<db_Table>(class_name.c_str(),
                                      "owner", schema.valueptr(),
                                      "name", grt::StringRef(name).valueptr(),
                                      NULL);

  _undo_manager->add_undo(new UndoListAddAction(schema.tables()));
  _undo_manager->set_action_description(_("Add New Table Object"));
  schema.tables().insert(table);

  return table;
}



void WBContextPhysical::add_new_db_table(const std::string &schema_id)
{
  grt::ValueRef res;

  res= execute_in_grt_thread("Create new table",
                        sigc::bind<std::string>(sigc::mem_fun(this, &WBContextPhysical::add_new_db_table_grt),
                                                            schema_id));

  if (res.is_valid())
  {
    db_TableRef table(db_TableRef::cast_from(res));

    show_status_text(fmt(_("Table '%s' created in schema '%s'"), table.name().c_str(),
                         table.owner().name().c_str()));

    open_object_editor(GrtObjectRef::cast_from(res));
  }
  else
    show_status_text(_("Could not create new table."));
}


grt::ValueRef WBContextPhysical::add_new_db_view_grt(grt::GRT *grt,
                                                         const std::string &schema_id)
{
  db_SchemaRef schema;
  db_ViewRef view;
  std::string class_name;

  schema= db_SchemaRef::cast_from(grt->find_object(schema_id, "/wb/doc/physicalModels"));
  if (!schema.is_valid())
    throw grt::grt_runtime_error("Cannot create view, target schema not found",
                                 "Schema '"+schema_id+"' was not found");

  class_name= *get_parent_for_object<workbench_physical_Model>(schema).rdbms().databaseObjectPackage()+".View";

  std::string name= grt::get_name_suggestion_for_list_object(grt::ObjectListRef::cast_from(schema.tables()), "view");

  view= grt->create_object<db_View>(class_name.c_str(),
                                    "owner", schema.valueptr(),
                                    "name", grt::StringRef(name).valueptr(),
                                    NULL);

  _undo_manager->add_undo(new UndoListAddAction(schema.views()));
  _undo_manager->set_action_description(_("Add New View Object"));

  schema.views().insert(view);

  return view;
}



void WBContextPhysical::add_new_db_view(const std::string &schema_id)
{
  grt::ValueRef res;

  res= execute_in_grt_thread("Create new view",
                        sigc::bind<std::string>(sigc::mem_fun(this, &WBContextPhysical::add_new_db_view_grt),
                                                            schema_id));
  if (res.is_valid())
  {
    db_ViewRef view(db_ViewRef::cast_from(res));

    show_status_text(fmt(_("View '%s' created in schema '%s'"), view.name().c_str(),
                         view.owner().name().c_str()));

    open_object_editor(GrtObjectRef::cast_from(res));
  }
  else
    show_status_text(_("Could not create new view"));
}




grt::ValueRef WBContextPhysical::add_new_db_routine_group_grt(grt::GRT *grt,
                                                         const std::string &schema_id)
{
  db_SchemaRef schema;
  db_RoutineGroupRef rgroup;
  std::string class_name;

  schema= db_SchemaRef::cast_from(grt->find_object(schema_id, "/wb/doc/physicalModels"));
  if (!schema.is_valid())
    throw grt::grt_runtime_error("Cannot create routine group, target schema not found",
                                 "Schema '"+schema_id+"' was not found");

  class_name= *get_parent_for_object<workbench_physical_Model>(schema).rdbms().databaseObjectPackage()+".RoutineGroup";

  std::string name= grt::get_name_suggestion_for_list_object(grt::ObjectListRef::cast_from(schema.tables()), "routines");

  rgroup= grt->create_object<db_RoutineGroup>(class_name.c_str(),
                                    "owner", schema.valueptr(),
                                    "name", grt::StringRef(name).valueptr(),
                                    NULL);

  _undo_manager->add_undo(new UndoListAddAction(schema.routineGroups()));
  _undo_manager->set_action_description(_("Add New Routine Group Object"));

  schema.routineGroups().insert(rgroup);

  return rgroup;
}



void WBContextPhysical::add_new_db_routine_group(const std::string &schema_id)
{
  grt::ValueRef res;

  res= execute_in_grt_thread("Create new routine group",
                        sigc::bind<std::string>(sigc::mem_fun(this, &WBContextPhysical::add_new_db_routine_group_grt),
                                                            schema_id));
  if (res.is_valid())
  {
    db_RoutineGroupRef rgroup(db_RoutineGroupRef::cast_from(res));

    show_status_text(fmt(_("Routine group '%s' created in schema '%s'"), rgroup.name().c_str(),
                         rgroup.owner().name().c_str()));

    open_object_editor(GrtObjectRef::cast_from(res));
  }
  else
    show_status_text(_("Could not create new routine group"));
}



grt::ValueRef WBContextPhysical::add_new_db_routine_grt(grt::GRT *grt, const std::string &schema_id)
{
  db_SchemaRef schema;
  db_RoutineRef routine;
  std::string class_name;

  schema= db_SchemaRef::cast_from(grt->find_object(schema_id, "/wb/doc/physicalModels"));
  if (!schema.is_valid())
    throw grt::grt_runtime_error("Cannot create routine, target schema not found",
                                 "Schema '"+schema_id+"' was not found");

  class_name= *get_parent_for_object<workbench_physical_Model>(schema).rdbms().databaseObjectPackage()+".Routine";

  std::string name= grt::get_name_suggestion_for_list_object(grt::ObjectListRef::cast_from(schema.tables()), "routine");

  routine= grt->create_object<db_Routine>(class_name.c_str(),
                                    "owner", schema.valueptr(),
                                    "name", grt::StringRef(name).valueptr(),
                                    NULL);

  _undo_manager->add_undo(new UndoListAddAction(schema.routines()));
  _undo_manager->set_action_description(_("Add New Routine Object"));

  schema.routines().insert(routine);

  return routine;
}



void WBContextPhysical::add_new_db_routine(const std::string &schema_id)
{
  grt::ValueRef res;

  res= execute_in_grt_thread("Create new routine",
                        sigc::bind<std::string>(sigc::mem_fun(this, &WBContextPhysical::add_new_db_routine_grt),
                                                            schema_id));
  if (res.is_valid())
  {
    db_RoutineRef routine(db_RoutineRef::cast_from(res));

    show_status_text(fmt(_("Routine '%s' created in schema '%s'"), routine.name().c_str(),
                         routine.owner().name().c_str()));

    open_object_editor(GrtObjectRef::cast_from(res));
  }
  else
    show_status_text(_("Could not create new routine"));
}




//--------------------------------------------------------------------------------
// Canvas Object Management


grt::ValueRef WBContextPhysical::place_db_object_grt(grt::GRT *grt, mdc::CanvasView *cview, const mdc::Point &pos, const db_DatabaseObjectRef &object)
{
  std::string object_member;
  std::string figure_struct_name;

  std::string object_type, object_struct_name;

  if (object.is_instance("db.Table"))
  {
    object_member= "table";
    figure_struct_name= workbench_physical_TableFigure::static_class_name();
  }
  else if (object.is_instance("db.View"))
  {
    object_member= "view";
    figure_struct_name= workbench_physical_ViewFigure::static_class_name();
  }
  else if (object.is_instance("db.RoutineGroup"))
  {
    object_member= "routineGroup";
    figure_struct_name= workbench_physical_RoutineGroupFigure::static_class_name();
  }
  else
    throw std::invalid_argument("trying to place invalid object on view");

  model_FigureRef figure(model_FigureRef::cast_from(place_object_grt(grt, get_id_of_view(cview), pos, figure_struct_name)));

  figure.set_member(object_member.c_str(), object);

  return figure;
}


void WBContextPhysical::place_db_object(mdc::CanvasView *view, const mdc::Point &pos, const db_DatabaseObjectRef &object)
{
  grt::ValueRef r;
  
  try {
    r= execute_in_grt_thread("Place existing object in model",
                        sigc::bind<mdc::CanvasView*,mdc::Point,db_DatabaseObject>(sigc::mem_fun(this, &WBContextPhysical::place_db_object_grt),
                                                                      view,
                                                                      pos,
                                                                      object));
  }
  catch (std::invalid_argument &)
  {
    show_status_text(_("Cannot place object."));
    return;
  }
  catch (grt::grt_runtime_error &exc)
  {
    show_exception(_("Please Object on Canvas"), exc);
    return;
  }

  if (r.is_valid())
    show_status_text(fmt(_("Placed %s"), GrtObjectRef::cast_from(r).name().c_str()));
  else
    show_status_text(_("Failed placing db object."));
}


void WBContextPhysical::place_db_object(mdc::CanvasView *view, int x, int y, const db_DatabaseObjectRef &object)
{
  place_db_object(view, view->window_to_canvas(x, y), object);
}


grt::ValueRef WBContextPhysical::place_new_db_object_grt(grt::GRT *grt, mdc::CanvasView *cview, 
                                                             const mdc::Point &pos,
                                                             ObjectType type)
{
  std::string object_struct_name;
  std::string schema_id, schema_name;

  model_ViewRef view= get_view_with_id(get_id_of_view(cview));
  workbench_physical_ModelRef model(get_parent_for_object<workbench_physical_Model>(view));

  switch (type)
  {
  case ObjectTable:
    object_struct_name= workbench_physical_TableFigure::static_class_name();
    break;
  case ObjectView:
    object_struct_name= workbench_physical_ViewFigure::static_class_name();
    break;
  case ObjectRoutineGroup:
    object_struct_name= workbench_physical_RoutineGroupFigure::static_class_name();
    break;
  default:
    throw std::logic_error("place_db_object() called with invalid tool");
  }
  schema_name= get_canvas_tool_argument(cview, object_struct_name+std::string(":Schema"));

  if (!schema_name.empty())
  {
    db_SchemaRef schema(grt::find_named_object_in_list(model.catalog().schemata(), schema_name));
    if (schema.is_valid())
      schema_id= schema.id();
  }

  // pick a default schema..
  if (schema_id.empty())
    schema_id= model.catalog().schemata().get(0).id();

  db_DatabaseObjectRef object;

  _undo_manager->begin_undo_group();

  switch (type)
  {
  case ObjectTable:
    object= db_DatabaseObjectRef::cast_from(add_new_db_table_grt(grt, schema_id));
    break;
  case ObjectView:
    object= db_DatabaseObjectRef::cast_from(add_new_db_view_grt(grt, schema_id));
    break;
  case ObjectRoutineGroup:
    object= db_DatabaseObjectRef::cast_from(add_new_db_routine_group_grt(grt, schema_id));
    break;
  default:
    throw std::logic_error("place_db_object() called with invalid tool");
  }

  std::string collation= get_canvas_tool_argument(cview, object_struct_name+":Collation");

  if (collation!="" && collation[0] != '*')
  {
    if (object.has_member("defaultCollationName"))
      object.set_member("defaultCollationName", collation);

    std::string charset= split_string(collation, "_", 1)[0];
    if (object.has_member("defaultCharacterSetName"))
      object.set_member("defaultCharacterSetName", charset);
  }

  grt::ValueRef result= place_db_object_grt(grt, cview, pos, object);

  _undo_manager->end_undo_group();
  _undo_manager->set_action_description(fmt(_("Place New %s"), object.get_attribute("caption").c_str()));

  return result;
}


void WBContextPhysical::place_new_db_object(mdc::CanvasView *view, const mdc::Point &pos,
                                            ObjectType type)
{
  grt::ValueRef r;
  
  r= execute_in_grt_thread("Place object in model",
                        sigc::bind<mdc::CanvasView*,mdc::Point,ObjectType>
                           (sigc::mem_fun(this, &WBContextPhysical::place_new_db_object_grt),
                            view,  pos, type));
  if (r.is_valid())
    show_status_text(fmt(_("Placed new %s"), model_FigureRef::cast_from(r).name().c_str()));
  else
    show_status_text(_("Failed placing db object."));
}


bool WBContextPhysical::create_nm_relationship_grt(grt::GRT *grt, mdc::CanvasView *view,
                                                   workbench_physical_TableFigureRef table1, 
                                                   workbench_physical_TableFigureRef table2,
                                                   bool imandatory, bool fmandatory)
{
  _undo_manager->begin_undo_group();
  // create the associative table for a n:m relationship
  db_TableRef atable=
    bec::TableHelper::create_associative_table(db_SchemaRef::cast_from(table1.table().owner()),
                                              table1.table(), table2.table(),
                                              imandatory, fmandatory, _undo_manager);

  if (!atable.is_valid())
  {
    _undo_manager->cancel_undo_group();
    return false;
  }

  // place the assoc table in the view
  mdc::Point pos;

  pos.x= (table1.left() + table2.left()) / 2;
  pos.y= (table1.top() + table2.top()) / 2;

  place_db_object_grt(grt, view, pos, atable);

  // force refresh of fks connections
  handle_fklist_change(atable, atable.foreignKeys().get(0), true);
  handle_fklist_change(atable, atable.foreignKeys().get(1), true);

  _undo_manager->end_undo_group();

  return true;
}


grt::ValueRef WBContextPhysical::update_relationship_grt(grt::GRT *grt, 
                                                            const std::string &view_id,
                                                            const mdc::Point &pos)
{
  // get the view for the view id
  model_ViewRef view(get_view_with_id(view_id));
  if (!view.is_valid())
  {
    throw std::logic_error("View with id "+view_id+" not found.");
  }

  grt::StringRef result;

  mdc::CanvasView *cview;
  cview= dynamic_cast<wbbridge::ViewBase*>(view.get_bridge_private())->get_canvas_view();

  switch (_relationship_context[cview].state)
  {
  case RIdle:
    break;

  case RPickingStart:
    {
      // check if there's a table at the clicked point
      model_ObjectRef obj= get_object_at(view, cview, pos);
      if (obj.is_valid() && obj.is_instance(workbench_physical_TableFigure::static_class_name()))
      {
        workbench_physical_TableFigureRef table(workbench_physical_TableFigureRef::cast_from(obj));

        if (_relationship_context[cview].type == RelationshipnmId &&
          !table.table().primaryKey().is_valid())
        {
          result= grt::StringRef::format(_("'%s' has no primary key. Please add a PK or select another table."),
                  table.table().name().c_str());
          break;
        }

        _relationship_context[cview].itable= table;
        
        dynamic_cast<wbbridge::physical::TableFigure*>(table.get_bridge_private())->set_highlighted(true);

        result= grt::StringRef::format(_("'%s' selected. Please select the referred table."), 
               table.table().name().c_str());

        _relationship_context[cview].state= RPickingEnd;
      }
      else
        result= grt::StringRef(_("Please select a table."));
    }
    break;

  case RPickingEnd:
    {
      // check if there's a table at the clicked point
      model_ObjectRef obj= get_object_at(view, cview, pos);
      if (obj.is_valid() && obj.is_instance(workbench_physical_TableFigure::static_class_name()))
      {
        workbench_physical_TableFigureRef table(workbench_physical_TableFigureRef::cast_from(obj));
        bool imany, fmany;
        bool imand, fmand;
        bool identifying= false;

        if (!table.table().primaryKey().is_valid())
        {
          result= grt::StringRef::format(_("'%s' has no primary key. Please add a PK or select another table."),
                  table.table().name().c_str());
          break;
        }

        imand= get_canvas_tool_argument(cview, "workbench.physical.Connection:optional")=="1";
        fmand= get_canvas_tool_argument(cview, "workbench.physical.Connection:refOptional")=="1";

        switch (_relationship_context[cview].type)
        {
        case Relationship11Id:    imany= false; fmany= false; identifying= true; break;
        case Relationship1nId:    imany= true;  fmany= false; identifying= true; break;
        case Relationship11NonId: imany= false; fmany= false; identifying= false; break;
        case Relationship1nNonId: imany= true;  fmany= false; identifying= false; break;
        case RelationshipnmId:    imany= true;  fmany= true;  break;
        default: break;
        }

        dynamic_cast<wbbridge::physical::TableFigure*>
          (_relationship_context[cview].itable.get_bridge_private())->set_highlighted(false);

        if (imany && fmany)
        {
          bool ok= create_nm_relationship_grt(grt, cview, _relationship_context[cview].itable, 
                                              table, imand, fmand);

          if (ok)
          {
            _undo_manager->set_action_description(_("Create Relationship"));

            result= grt::StringRef::format(_("Relationship between '%s' and '%s' created through an associative table."),
                                             _relationship_context[cview].itable.table().name().c_str(),
                                             table.table().name().c_str());
          }
          else
            result= grt::StringRef::format(_("Cannot create relationship between '%s' and '%s'."), 
                                             _relationship_context[cview].itable.table().name().c_str(),
                                             table.table().name().c_str());
        }
        else
        {
          // create the FK, the relationship will be auto-created as a side-effect
          db_ForeignKeyRef fk;

          fk= bec::TableHelper::create_foreign_key_to_table(_relationship_context[cview].itable.table(),
                                                            table.table(),
                                                            imand, fmand, imany, identifying,
                                                            get_wb_options(),
                                                            _undo_manager);
          if (fk.is_valid())
          {
            _undo_manager->set_action_description(_("Create Relationship"));

            result= grt::StringRef::format(_("Relationship between '%s' and '%s' created."), 
                                             _relationship_context[cview].itable.table().name().c_str(), 
                                             table.table().name().c_str());
          }
          else
          {
            result= grt::StringRef::format(_("Cannot create relationship between '%s' and '%s'."), 
                                             _relationship_context[cview].itable.table().name().c_str(),
                                             table.table().name().c_str());
          }
        }

        _relationship_context[cview].state= RFinished;
      }
      else
        result= grt::StringRef(_("Please select the referred table."));
    }
    break;

  case RFinished:
    break;

  case RCancelled:
    break;
  }

  return result;
}


void WBContextPhysical::start_relationship(mdc::CanvasView *cview, const mdc::Point &pos,
                                           RelationshipType type)
{
  grt::ValueRef result;

  _relationship_context[cview].type= type;
  _relationship_context[cview].state= RPickingStart;
  _relationship_context[cview].last_message= _("Please select the starting table for the relationship.");
  show_status_text(_relationship_context[cview].last_message);
}


bool WBContextPhysical::update_relationship(mdc::CanvasView *cview, const mdc::Point &pos)

{
  grt::ValueRef result;

  switch (_relationship_context[cview].state)
  {
  case RPickingStart:
  case RPickingEnd: 
    result= execute_in_grt_thread("Place relationship in model",
                          sigc::bind<std::string,mdc::Point>
                                  (sigc::mem_fun(this, &WBContextPhysical::update_relationship_grt),
                                   get_id_of_view(cview), pos));
    if (result.is_valid())
    {
      _relationship_context[cview].last_message= *grt::StringRef::cast_from(result);
      show_status_text(_relationship_context[cview].last_message);
    }

    if (_relationship_context[cview].state == RFinished)
      return true;

    break;
  
  case RFinished:
    return true;

  case RCancelled:
    return true;
    
  default:
    return false;
  }
  
  return false;
}


void WBContextPhysical::cancel_relationship(mdc::CanvasView *cview)
{
  if (_relationship_context.find(cview) != _relationship_context.end())
  {
    if (_relationship_context[cview].itable.is_valid())
        dynamic_cast<wbbridge::physical::TableFigure*>
          (_relationship_context[cview].itable.get_bridge_private())->set_highlighted(false);

    _relationship_context.erase(cview);

    show_status_text(_("Cancelled."));
  }  
}


void WBContextPhysical::delete_db_object(const db_DatabaseObjectRef &object)
{
  db_SchemaRef schema(db_SchemaRef::cast_from(object.owner()));
  workbench_physical_ModelRef model(get_parent_for_object<workbench_physical_Model>(schema));

  //XXX need to look for refs by other objects and show them to user
  // and confirm that they should be removed or cancel

  if (object.is_instance(db_Table::static_class_name()))
  {
    _undo_manager->begin_undo_group();
    _undo_manager->add_undo(new UndoListRemoveAction(schema.tables(), object));

    schema.tables().remove_value(db_TableRef::cast_from(object));

    if (model.is_valid())
    {
      for (size_t vc= model.views().count(), v= 0; v < vc; v++)
      {
        model_ViewRef view(model.views().get(v));

        for (ssize_t f= view.figures().count()-1; f >= 0; --f)
        {
          model_FigureRef figure(view.figures().get(f));

          if (figure.is_instance(workbench_physical_TableFigure::static_class_name())
            && workbench_physical_TableFigureRef::cast_from(figure).table() == object)
          {
            delete_model_object(figure, true);
          }
        }
      }
    }

    _undo_manager->end_undo_group();
    _undo_manager->set_action_description(_("Delete Table"));
  }
  else if (object.is_instance(db_View::static_class_name()))
  {
    _undo_manager->begin_undo_group();
    _undo_manager->add_undo(new UndoListRemoveAction(schema.views(), object));

    schema.views().remove_value(db_ViewRef::cast_from(object));

    if (model.is_valid())
    {
      for (size_t vc= model.views().count(), v= 0; v < vc; v++)
      {
        model_ViewRef view(model.views().get(v));

        for (ssize_t f= view.figures().count()-1; f >= 0; --f)
        {
          model_FigureRef figure(view.figures().get(f));

          if (figure.is_instance(workbench_physical_ViewFigure::static_class_name())
            && workbench_physical_ViewFigureRef::cast_from(figure).view() == object)
          {
            delete_model_object(figure, true);
          }
        }
      }
    }
    _undo_manager->end_undo_group();
    _undo_manager->set_action_description(_("Delete View"));
  }
  else if (object.is_instance(db_RoutineGroup::static_class_name()))
  {
    _undo_manager->begin_undo_group();
    _undo_manager->add_undo(new UndoListRemoveAction(schema.routineGroups(), object));
    
    schema.routineGroups().remove_value(db_RoutineGroupRef::cast_from(object));

    if (model.is_valid())
    {
      for (size_t vc= model.views().count(), v= 0; v < vc; v++)
      {
        model_ViewRef view(model.views().get(v));

        for (ssize_t f= view.figures().count()-1; f >= 0; --f)
        {
          model_FigureRef figure(view.figures().get(f));

          if (figure.is_instance(workbench_physical_RoutineGroupFigure::static_class_name())
            && workbench_physical_RoutineGroupFigureRef::cast_from(figure).routineGroup() == object)
          {
            delete_model_object(figure, true);
          }
        }
      }
    }
    _undo_manager->end_undo_group();
    _undo_manager->set_action_description(_("Delete Routine Group"));
  }
  else if (object.is_instance(db_Routine::static_class_name()))
  {
    _undo_manager->add_undo(new UndoListRemoveAction(schema.routines(), object));
    _undo_manager->set_action_description(_("Delete Routine"));

    schema.routines().remove_value(db_RoutineRef::cast_from(object));
    //XXX remove from routine groups
  }
}



bool WBContextPhysical::delete_physical_model_object(const model_ObjectRef &object, bool view_only)
{
  if (object.is_instance(workbench_physical_Connection::static_class_name()))
  {
    workbench_physical_ConnectionRef conn(workbench_physical_ConnectionRef::cast_from(object));

    _undo_manager->begin_undo_group();

    if (!view_only)
    {
      db_ForeignKeyRef fk(conn.foreignKey());

      // delete foreign key from table
      _undo_manager->add_undo(new UndoListRemoveAction(db_TableRef::cast_from(fk.owner()).foreignKeys(), fk));
      db_TableRef::cast_from(fk.owner()).foreignKeys().remove_value(fk);
    }

    _undo_manager->add_undo(new UndoObjectChangeAction(conn, "startFigure"));
    conn.startFigure(model_Figure());

    _undo_manager->add_undo(new UndoObjectChangeAction(conn, "endFigure"));
    conn.endFigure(model_Figure());

    // now try to remove all references to the figure
    _undo_manager->add_undo(new UndoListRemoveAction(conn.owner().connections(), conn));
    conn.owner().connections().remove_value(conn);

    _undo_manager->end_undo_group();
    _undo_manager->set_action_description(_("Delete Relationship"));

  }
  else if (object.is_instance(model_Figure::static_class_name()))
  {
    model_FigureRef figure(model_FigureRef::cast_from(object));

    // if the figure is a DB object, the DB object is also deleted
    if (figure.is_instance(workbench_physical_TableFigure::static_class_name()))
    {
      _undo_manager->begin_undo_group();
      if (!view_only)
        delete_db_object(workbench_physical_TableFigureRef::cast_from(figure).table());

      _undo_manager->add_undo(new UndoObjectChangeAction(figure, "table"));
      workbench_physical_TableFigureRef::cast_from(figure).table(db_Table());
    }
    else if (figure.is_instance(workbench_physical_ViewFigure::static_class_name()))
    {
      _undo_manager->begin_undo_group();
      if (!view_only)
        delete_db_object(workbench_physical_ViewFigureRef::cast_from(figure).view());

      _undo_manager->add_undo(new UndoObjectChangeAction(figure, "view"));
      workbench_physical_ViewFigureRef::cast_from(figure).view(db_View());
    }
    else if (figure.is_instance(workbench_physical_RoutineGroupFigure::static_class_name()))
    {
      _undo_manager->begin_undo_group();
      if (!view_only)
        delete_db_object(workbench_physical_RoutineGroupFigureRef::cast_from(figure).routineGroup());

      _undo_manager->add_undo(new UndoObjectChangeAction(figure, "routineGroup"));
      workbench_physical_RoutineGroupFigureRef::cast_from(figure).routineGroup(db_RoutineGroup());
    }
    else
      return false;
    
    // now try to remove all references to the figure
    _undo_manager->add_undo(new UndoListRemoveAction(figure.layer().figures(), figure));
    _undo_manager->add_undo(new UndoListRemoveAction(model_ViewRef::cast_from(figure.owner()).figures(), figure));

    figure.layer().figures().remove_value(figure);
    model_ViewRef::cast_from(figure.owner()).figures().remove_value(figure);

    _undo_manager->end_undo_group();
    _undo_manager->set_action_description(fmt(_("Delete '%s' Figure"), figure.get_attribute("caption").c_str()));
  }
  return true;
}



void WBContextPhysical::activate_figure(const model_FigureRef &figure)
{
  if (workbench_physical_TableFigureRef::can_wrap(figure))
    WBContextBase::open_object_editor(workbench_physical_TableFigureRef::cast_from(figure).table());

  else if (workbench_physical_ViewFigureRef::can_wrap(figure))
    WBContextBase::open_object_editor(workbench_physical_ViewFigureRef::cast_from(figure).view());

  else if (workbench_physical_RoutineGroupFigureRef::can_wrap(figure))
    WBContextBase::open_object_editor(workbench_physical_RoutineGroupFigureRef::cast_from(figure).routineGroup());
}



//--------------------------------------------------------------------------------



grt::ListRef<app_ToolbarItem> WBContextPhysical::get_toolbar_items_for_view(const std::string &toolbar)
{
  if (toolbar == WB_TOOLBAR_MAIN)
  {
    if (_toolbars.find(toolbar+"/"+get_canvas_tool(get_active_canvas_view())) == _toolbars.end())
      return grt::ListRef<app_ToolbarItem>();
    return _toolbars[toolbar+"/"+get_canvas_tool(get_active_canvas_view())].items();
  }
  else
    return _toolbars[toolbar].items();
}


std::vector<std::string> WBContextPhysical::get_dropdown_items(const std::string &option, std::string &selected)
{
  std::vector<std::string> items;
  if (has_prefix(option, "workbench.physical."))
  {
    if (has_suffix(option, ":Color"))
    {
      items.push_back("#98bfda");
      items.push_back("#fede58");
      items.push_back("#98d8a5");

      items.push_back("#fe9898");
      items.push_back("#fe98fe");

      items.push_back("#ffffff");

      selected= get_canvas_tool_argument(get_active_canvas_view(), option);
      if (selected.empty())
        selected= get_wb_options().get_string(option);
      if (selected.empty())
        selected= items[0];
      if (!selected.empty() && std::find(items.begin(), items.end(), selected) == items.end())
        items.push_back(selected);
    }
    else if (has_suffix(option, ":Schema"))
    {
      workbench_physical_ModelRef model(get_parent_for_object<workbench_physical_Model>(get_active_view()));

      if (model.is_valid())
      {
        for (size_t c= model.catalog().schemata().count(), i= 0; i < c; i++)
          items.push_back(*model.catalog().schemata().get(i).name());
        std::sort(items.begin(), items.end());
      }

      selected= get_canvas_tool_argument(get_active_canvas_view(), option);
      if (selected.empty() || std::find(items.begin(), items.end(), selected) == items.end())
        selected= items[0];
    }
    else if (has_suffix(option, ":Engine"))
    {
      items.push_back("*Default*");

      selected= get_canvas_tool_argument(get_active_canvas_view(), option);
      if (selected.empty())
        selected= get_wb_options().get_string(option);
      if (selected.empty())
        selected= "*Default*";
    }
    else if (has_suffix(option, ":Collation"))
    {
      workbench_physical_ModelRef model(get_parent_for_object<workbench_physical_Model>(get_active_view()));

      if (_collation_list.empty())
      {
        items.push_back("*Default*");
        if (model.is_valid())
        {
          for (size_t c= model.catalog().characterSets().count(), i= 0; i < c; i++)
          {
            db_CharacterSetRef charset(model.catalog().characterSets().get(i));

            for (size_t d= charset.collations().count(), j= 0; j < d; j++)
              items.push_back(*charset.collations().get(j));
          }
          std::sort(items.begin(), items.end());
        }
        _collation_list= items;
      }
      else
        items= _collation_list;

      selected= get_canvas_tool_argument(get_active_canvas_view(), option);
      if (selected.empty())
        selected= get_wb_options().get_string(option);
      if (selected.empty())
        selected= "*Default*";
    }
  }
  else
    throw std::logic_error("Unknown option "+option);

  return items;
}


//--------------------------------------------------------------------------------
// Object Listeners

void WBContextPhysical::reset_schemata_listeners()
{
  grt::GRT *grt= _manager->get_grt();

  if (_schemata_listener > 0)
    grt->remove_listener(_schemata_listener);

  workbench_DocumentRef doc(get_document());

  for (size_t c= doc.physicalModels().count(), i= 0; i < c; i++)
  {
    workbench_physical_ModelRef model(doc.physicalModels()[i]);

    if (model.is_valid() && model.catalog().is_valid() && model.catalog().schemata().is_valid())
      _schemata_listener= grt->add_listener(model.catalog().schemata(), NULL, 
                          sigc::mem_fun(this, &WBContextPhysical::schema_list_changed));
  }
}

/** 
 ****************************************************************************
 * @brief Listener for changes in the list of schemas
 *
 * Used for attaching listeners to new schemas and content lists.
 *
 ****************************************************************************
 */
void WBContextPhysical::schema_list_changed(MYX_GRT_NOTIFICATION *notif)
{
  // we're called in the GRT thread, so just mark the refresh request
  // as pending. This has the bonus that multiple requests will be
  // collapsed into 1.
  // The requests are actually sent in flush_idle_tasks()

  request_refresh(RefreshSchemaList, "", 0);


  if (notif->name == myx_notify_list_added)
  {
    db_SchemaRef schema(db_SchemaRef::cast_from(grt::DictRef(notif->argdata).get("value")));
    grt::GRT *grt= _manager->get_grt();

    // added new schema

    // listener for changes in schema itself
    grt->add_listener(schema,
                      myx_notify_object_changed,
                      sigc::mem_fun(this, &WBContextPhysical::schema_changed));
    
    // for changes in table list
    grt->add_listener(schema.tables(), NULL,
                      sigc::mem_fun(this, &WBContextPhysical::schema_object_list_changed));
    
    // in view list
    grt->add_listener(schema.views(), NULL,
                      sigc::mem_fun(this, &WBContextPhysical::schema_object_list_changed));

    // in SP/function list
    grt->add_listener(schema.routines(), NULL,
                      sigc::mem_fun(this, &WBContextPhysical::schema_object_list_changed));

    // in routine groups list
    grt->add_listener(schema.routineGroups(), NULL,
                      sigc::mem_fun(this, &WBContextPhysical::schema_object_list_changed));
  }
  else if (notif->name == myx_notify_list_removed)
  {
    //XXX TODO
  }
  else if (notif->name == myx_notify_list_cleared)
  {
    //XXX TODO
  }
}



void WBContextPhysical::schema_changed(MYX_GRT_NOTIFICATION *notif)
{
  request_refresh(RefreshSchema, "", 0);
}


void WBContextPhysical::schema_object_list_changed(MYX_GRT_NOTIFICATION *notif)
{
  request_refresh(RefreshSchema, "", 0);
}


void WBContextPhysical::db_object_changed(MYX_GRT_NOTIFICATION *notif)
{
  const char *key= myx_grt_dict_item_get_as_string(notif->argdata, "key");

  if (key && strcmp(key, "name")==0)
  {
    request_refresh(RefreshSchemaList, "", 0);
  }
}


/** 
 ****************************************************************************
 * @brief Bridge delegate for changes in foreign key list
 *
 * This is called by the bridge whenever the fk list for a table is changed.
 * It will check for removed or created relationships (connections) and
 * update them accordingly.
 *
 * Do not call directly.
 *
 * @param table
 * @param fk fk that was added or removed, if !valid and !added then the list
 * was cleared
 * @param added whether the fk was added or removed from the list
 *
 ****************************************************************************
 */
void WBContextPhysical::handle_fklist_change(const db_TableRef &table, const db_ForeignKeyRef &fk, bool added)
{
  workbench_physical_ModelRef model(get_parent_for_object<workbench_physical_Model>(table));

  if (!model.is_valid() || !model.views().is_valid())
    return;

  if (!fk.is_valid())
  {
    if (!added)
    {
      // all FKs from table removed
    }
  }
  else
  {
    if (!fk.referedTable().is_valid() || !fk.owner().is_valid())
      return;
    
    grt::ListRef<workbench_physical_View> views(model.views());

    if (added)
    {
      // we have to go through all views in the model and find all table figures
      // that correspond to the FK for creating the relationship in all these views

      for (size_t c= views.count(), i= 0; i < c; i++)
      {
        grt::ListRef<model_Figure> figures(views[i]->figures());
        workbench_physical_TableFigureRef table1, table2;

        for (size_t d= figures.count(), f= 0; f < d; f++)
        {
          model_FigureRef fig(figures[f]);
          
          if (fig.is_instance(workbench_physical_TableFigure::static_class_name()))
          {
            workbench_physical_TableFigureRef tablefig(workbench_physical_TableFigureRef::cast_from(fig));
            
            if (tablefig.table() == table)
            {
              table1= tablefig;
              if (table2.is_valid())
                break;
            }
            if (tablefig.table() == fk.referedTable())
            {
              table2= tablefig;
              if (table1.is_valid())
                break;
            }
          }
        }
        if (table1.is_valid() && table2.is_valid())
        { // both tables in the relationship are in this view, so create the connection
           
          // but 1st check if it already exists
          grt::ListRef<model_Connection> connections(views[i]->connections());
          bool found= false;

          for (size_t d= connections.count(), j= 0; j < d; j++)
          {
            model_ConnectionRef conn(connections[j]);
              
            if (conn.is_instance(workbench_physical_Connection::static_class_name()))
            {
              workbench_physical_ConnectionRef pconn(workbench_physical_ConnectionRef::cast_from(conn));

              if (pconn.foreignKey() == fk)
              {
                found= true;
                break;
              }
            }
          }

          // connection doesnt exist yet, create it
          if (!found)
          {
            grt::GRT *grt= table.get_grt();
              
            workbench_physical_ConnectionRef conn(grt->create_object<workbench_physical_Connection>
                                               (0,
                                                "owner", views[i]->valueptr(), 
                                                "startFigure", table1.valueptr(),
                                                "endFigure", table2.valueptr(),
                                                "caption", fk.name().valueptr(),
                                                "foreignKey", fk.valueptr(),
                                                NULL));

            views[i]->connections().insert(conn);
          }
        }
      }
    }
    else
    {
      // we have to go through all views in the model and find all connections
      // that correspond to the FK for removing them

      for (size_t c= views.count(), i= 0; i < c; i++)
      {
        grt::ListRef<model_Connection> connections(views[i]->connections());

        for (ssize_t j= connections.count()-1; j >= 0; --j)
        {
          model_ConnectionRef conn(connections[j]);

          if (conn.is_instance(workbench_physical_Connection::static_class_name()))
          {
            workbench_physical_ConnectionRef pconn(workbench_physical_ConnectionRef::cast_from(conn));
            
            if (pconn.foreignKey() == fk)
            {
              // remove this connection
              connections.remove_value(pconn);
            }
          }
        }
      }
    }
  }
}


/** 
 ****************************************************************************
 * @brief Bridge delegate for database object figure changes.
 *
 * This is called by the bridge whenever a db object has a canvas figure 
 * created for it. The handler will store a mapping of db objects to 
 * figures for use in other places.
 * 
 * Do not call directly.
 * 
 * @param object the db object
 * @param figure the model figure that was assigned to the object
 * @param attach whether the figure was attached to the object or not
 *
 ****************************************************************************
 */
void WBContextPhysical::notify_dbobject_figure(const db_DatabaseObjectRef &object, const model_FigureRef &figure, bool attach)
{
  std::multimap<std::string,model_Figure>::iterator next, iter= _dbobject_figures.find(object.id());

  if (attach)
  {
    while (iter != _dbobject_figures.end() && iter->first == object.id())
    {
      if (iter->second == figure) // already known
        return;
      ++iter;
    }

    _dbobject_figures.insert(std::pair<std::string,model_Figure>(object.id(), figure));
  }
  else
  {
    while (iter != _dbobject_figures.end() && iter->first == object.id())
    {
      next= iter;
      ++next;

      if (iter->second == figure) // found
        _dbobject_figures.erase(iter);
      iter= next;
    }
  }
}


bool WBContextPhysical::filter_catalog_tree(const NodeId &node, const std::string &key, 
                                            const grt::ValueRef &value, 
                                            std::string &label_ret,
                                            IconId &icon_ret)
{
  if (node.depth() < 2)
    return true;
  else if (node.depth() == 2)
  {
    if (key == "tables")
    {
      label_ret= _("Tables");
      return true;
    }
    else if (key == "views")
    {
      label_ret= _("Views");
      return true;
    }
    else if (key == "routines")
    {
      label_ret= _("Routines");
      return true;
    }
    else if (key == "routineGroups")
    {
      label_ret= _("Routine Groups");
      return true;
    }
    return false;
  }
  else if (node.depth() == 3)
  {
    GrtObjectRef object(GrtObjectRef::cast_from(value));

    if (_dbobject_figures.find(object.id()) != _dbobject_figures.end())
      label_ret= *object.name()+" ";
    else
      label_ret= *object.name();

    return true;
  }
  else
    return false;
}


std::string WBContextPhysical::get_catalog_path_for_view(mdc::CanvasView *cview)
{
  std::string view_id= get_id_of_view(cview);

  // look in the physicalModels list for this view
  
  for (size_t c= get_document().physicalModels().count(), i= 0; i < c; i++)
  {
    workbench_physical_ModelRef model= get_document().physicalModels()[i];

    for (size_t d= model.views().count(), j= 0; j < d; j++)
    {
      if (model.views().get(j).id() == view_id)
        return fmt("/wb/doc/physicalModels/%i/catalog", i);
    }
  }
  return "";
}
  


bec::ValueTreeBE *WBContextPhysical::get_catalog_tree(const std::string &path)
{
  if (!_catalog_tree)
  {
    _catalog_tree= new bec::ValueTreeBE(_manager->get_grt());
    _catalog_tree->set_node_filter(sigc::mem_fun(this, &WBContextPhysical::filter_catalog_tree));
  }

  if (*path.rbegin() == '/' && path != "/")
    _catalog_tree->set_displayed_global_value(path.substr(0, path.size()-1), false);
  else
    _catalog_tree->set_displayed_global_value(path, true);

  return _catalog_tree;
}


void WBContextPhysical::set_toolbar_option(const std::string &option, const std::string &item)
{
  set_canvas_tool_argument(get_active_canvas_view(), option, item);
}


void WBContextPhysical::set_canvas_tool(mdc::CanvasView *view, const std::string &tool)
{
  if (tool == WB_TOOL_PTABLE)
  {
    _view_data[view].cursor= "table";
    show_status_text(_("Select location for new table."));
  }
  else if (tool == WB_TOOL_PVIEW)
  {
    _view_data[view].cursor= "view";
    show_status_text(_("Select location for new view."));
  }
  else if (tool == WB_TOOL_PROUTINEGROUP)
  {
    _view_data[view].cursor= "routine";
    show_status_text(_("Select location for new routine group."));
  }
  else if (tool == WB_TOOL_PREL11)
  {
    _view_data[view].cursor= "rel11";
    start_relationship(view, mdc::Point(), Relationship11Id);
  }
  else if (tool == WB_TOOL_PREL1n)
  {
    _view_data[view].cursor= "rel1n";
    start_relationship(view, mdc::Point(), Relationship1nId);
  }
  else if (tool == WB_TOOL_PRELnm)
  {
    _view_data[view].cursor= "relnm";
    start_relationship(view, mdc::Point(), RelationshipnmId);
  }
  else if (tool == WB_TOOL_PREL11_NOID)
  {
    _view_data[view].cursor= "rel11";
    start_relationship(view, mdc::Point(), Relationship11NonId);
  }
  else if (tool == WB_TOOL_PREL1n_NOID)
  {
    _view_data[view].cursor= "rel1n";
    start_relationship(view, mdc::Point(), Relationship1nNonId);
  }
  else
  {
    show_status_text("Invalid tool "+tool);
    return;
  }
  _view_data[view].handle_button= sigc::mem_fun(this, &WBContextPhysical::handle_button_event);
  _view_data[view].handle_motion.disconnect();
  _view_data[view].reset_tool= sigc::mem_fun(this, &WBContextPhysical::cancel_relationship);
}



bool WBContextPhysical::handle_button_event(mdc::CanvasView *view, mdc::MouseButton button, bool press, 
                                      mdc::Point pos, mdc::EventState)
{
  std::string tool= get_canvas_tool(view);

  if (button != mdc::ButtonLeft)
    return false;

  if (tool == WB_TOOL_PTABLE)
  {
    if (press)
    {
      place_new_db_object(view, pos, ObjectTable);
      reset_canvas_tool(view, true);
      return true;
    }
  }
  else if (tool == WB_TOOL_PROUTINEGROUP)
  {
    if (press)
    {
      place_new_db_object(view, pos, ObjectRoutineGroup);
      reset_canvas_tool(view, true);
      return true;
    }
  }
  else if (tool == WB_TOOL_PVIEW)
  {
    if (press)
    {
      place_new_db_object(view, pos, ObjectView);
      reset_canvas_tool(view, true);
      return true;
    }
  }
  else if (tool == WB_TOOL_PREL11 || tool == WB_TOOL_PREL1n || tool == WB_TOOL_PRELnm ||
    tool == WB_TOOL_PREL11_NOID || tool == WB_TOOL_PREL1n_NOID)
  {
    if (press)
    {
      if (update_relationship(view, pos))
        reset_canvas_tool(view, true);
      return true;
    }
  }

  return false;
}
