
#include "gtk_helpers.h"
#include "image_cache.h"

#include "config.h"

#ifdef HAVE_LIBGNOME
#include <libgnome/libgnome.h>
#endif

// This list_model is used for all functions which operate on GTKListStore
struct ListColumnsModel : public Gtk::TreeModelColumnRecord
{
  public:
    ListColumnsModel() {add(item); }
    Gtk::TreeModelColumn<std::string>  item;
};

static ListColumnsModel _wb_list_model;

//------------------------------------------------------------------------------
Gtk::HBox &create_icon_label(const std::string &icon, const std::string &text)
{
  Gtk::HBox *hbox= Gtk::manage(new Gtk::HBox(false, 0));
  
  Gtk::Image *image= Gtk::manage(new Gtk::Image(ImageCache::get_instance()->image_from_filename(icon)));
  Gtk::Label *label= Gtk::manage(new Gtk::Label(text));
  
  label->set_use_markup(true);
  
  hbox->pack_start(*image);
  hbox->pack_start(*label, true, true);
  
  hbox->show_all();
  
  return *hbox;
}

//------------------------------------------------------------------------------
void open_url(const std::string &url)
{
#ifdef HAVE_LIBGNOME
  gnome_url_show(url.c_str(), NULL);
#else
  g_message("Open URL %s (no url handler)", url.c_str());
#endif
}

//------------------------------------------------------------------------------
Glib::RefPtr<Gtk::ListStore> model_from_string_list(const std::vector<std::string>& list)
{
  Glib::RefPtr<Gtk::ListStore> model = Gtk::ListStore::create(_wb_list_model);
  
  std::vector<std::string>::const_iterator last = list.end();
  
  for (std::vector<std::string>::const_iterator iter = list.begin(); iter != last; ++iter)
    (*model->append())[_wb_list_model.item] = *iter;
  
  return model;
}

//------------------------------------------------------------------------------
void recreate_model_from_string_list(Glib::RefPtr<Gtk::ListStore> model, const std::vector<std::string>& list)
{
  model->clear();
  
  std::vector<std::string>::const_iterator last = list.end();
  
  for (std::vector<std::string>::const_iterator iter = list.begin(); iter != last; ++iter)
    (*model->append())[_wb_list_model.item] = *iter;
}

//------------------------------------------------------------------------------
void set_glib_string(Glib::ValueBase& value, const std::string& str)
{
  GValue *gval = value.gobj();

  g_value_init(gval, G_TYPE_STRING);
  g_value_set_string(gval, str.c_str());
}

//------------------------------------------------------------------------------
void set_glib_int(Glib::ValueBase& value, const int i)
{
  GValue *gval = value.gobj();

  g_value_init(gval, G_TYPE_INT);
  g_value_set_int(gval, i);
}

//------------------------------------------------------------------------------
void set_glib_bool(Glib::ValueBase& value, const bool b)
{
  GValue *gval = value.gobj();

  g_value_init(gval, G_TYPE_BOOLEAN);
  g_value_set_boolean(gval, b);
}

//------------------------------------------------------------------------------
void set_glib_double(Glib::ValueBase& value, const double d)
{
  GValue *gval = value.gobj();

  g_value_init(gval, G_TYPE_DOUBLE);
  g_value_set_double(gval, d);
}

//------------------------------------------------------------------------------
void fill_combo_from_string_list(Gtk::ComboBoxText* combo, const std::vector<std::string>& list)
{
  std::vector<std::string>::const_iterator it   = list.begin();
  std::vector<std::string>::const_iterator last = list.end();

  for (; last != it; ++it )
    combo->append_text(*it);
}

//------------------------------------------------------------------------------
void fill_combo_from_string_list(Gtk::ComboBoxEntryText* combo, const std::vector<std::string>& list)
{
  std::vector<std::string>::const_iterator it   = list.begin();
  std::vector<std::string>::const_iterator last = list.end();

  for (; last != it; ++it )
    combo->append_text(*it);
}

/*
struct ItemFinder
{
  ItemFinder(const std::string& str) : pos (-1), index(-1), item(str) {};
  
  bool compare(const Gtk::TreeModel::Path&, const Gtk::TreeModel::iterator& iter)
  {
    ++pos;
    
    const Gtk::TreeModel::Row row = *iter;
    const std::string curitem = row[list_model.item];
    bool ret = curitem == item;
    
    if ( ret )
      index = pos;
    
    return ret;
  }

  int                 pos;
  int                 index;
  const std::string&  item;
};

//------------------------------------------------------------------------------
int item_index_in_model(Glib::RefPtr<Gtk::TreeModel> store, const std::string& item)
{
  ItemFinder finder(item);
  store->foreach(sigc::mem_fun(&finder, &ItemFinder::compare));
  //Gtk::TreeModel::Children::const_iterator it   = store.children().begin();
  //Gtk::TreeModel::Children::const_iterator last = store.children().end();
  
  return finder.index;
}

//------------------------------------------------------------------------------
std::string  get_active_item_from_combo(Gtk::ComboBox* combo)
{
  Gtk::TreeModel::Row row = *combo->get_active();
  return row[list_model.item];
}
*/
