//
//  MCanvasViewer.mm
//  MySQLWorkbench
//
//  Created by Alfredo Kojima on 13/Oct/08.
//  Copyright 2008 Sun Microsystems Inc. All rights reserved.
//

#import "sigobjc++.h"
#import "MCanvasViewer.h"
#include "mdc.h"
#include "mdc_canvas_view_macosx.h"


@implementation MCanvasViewer

- (id)initWithFrame:(NSRect)frame 
{
  self = [super initWithFrame:frame];
  if (self) 
  {
    _view= 0;
    
    _windowWasAcceptingMouseMoved= NO;
  }
  return self;
}


- (void)dealloc
{
  [_cursor release];
  _view->pre_destroy();
  delete _view;
  [super dealloc];
}


- (BOOL)isFlipped
{
  return YES;
}


- (void)drawRect:(NSRect)rect 
{
  NSRect frame= [self frame];
  int w, h;
  _view->get_view_size(w, h);
  
  if (w != (int)NSWidth(frame) || h != (int)NSHeight(frame))
    _view->update_view_size((int)NSWidth(frame), (int)NSHeight(frame));

  _view->repaint(rect.origin.x, rect.origin.y, rect.size.width, rect.size.height);
}


static void canvas_view_needs_repaint(int x, int y, int w, int h, MCanvasViewer *self)
{
  [self setNeedsDisplayInRect:NSMakeRect(x, y, w, h)];
}


- (void)setCursor:(NSCursor*)cursor
{
  [cursor retain];
  [_cursor release];
  _cursor= cursor;
  
  [[self window] resetCursorRects];
}


- (void)resetCursorRects
{
  [super resetCursorRects];
  if (_cursor)
  {
    [self addCursorRect:[self bounds] cursor:_cursor];
    [_cursor setOnMouseEntered:YES];
  }
}


- (void)setupQuartz
{
  NSRect frame= [self frame];
  _view= new mdc::QuartzCanvasView((CGContextRef)[[NSGraphicsContext currentContext] graphicsPort], 
                                   NSWidth(frame), NSHeight(frame));
  
  _view->signal_repaint().connect(sigc::bind(sigc::ptr_fun(canvas_view_needs_repaint), self));
}


- (mdc::CanvasView*)canvas
{
  return _view;
}


- (void)scrollToPoint:(NSPoint)offset
{
  _view->set_offset(mdc::Point(offset.x, offset.y));
}


- (NSRect)documentRect
{
  mdc::Size size(_view->get_total_view_size());

  return NSMakeRect(0, 0, size.width, size.height);
}


- (NSRect)documentVisibleRect
{
  mdc::Rect rect(_view->get_viewport());

  return NSMakeRect(rect.pos.x, rect.pos.y, rect.size.width, rect.size.height);
}


- (id)delegate
{
  return _delegate;
}

- (void)setDelegate:(id)delegate
{
  _delegate= delegate;
}


- (BOOL)acceptsFirstResponder
{
  return YES;
}

- (BOOL)becomeFirstResponder
{
  _windowWasAcceptingMouseMoved= [[self window] acceptsMouseMovedEvents];
  [[self window] setAcceptsMouseMovedEvents: YES];
  //_canvas->focusedSet(true);
  return YES;
}

- (BOOL)resignFirstResponder
{
  //_canvas->focusedSet(false);
  [[self window] setAcceptsMouseMovedEvents: _windowWasAcceptingMouseMoved];
  return YES;
}


static mdc::EventState makeEventState(NSEvent *event)
{
  int state= 0;

  if ([event modifierFlags] & NSShiftKeyMask)
    state|= mdc::SShiftMask;
  if ([event modifierFlags] & NSControlKeyMask)
    state|= mdc::SControlMask;
  if ([event modifierFlags] & NSAlternateKeyMask)
    state|= mdc::SOptionMask;
  if ([event modifierFlags] & NSCommandKeyMask)
    state|= mdc::SCommandMask;

  return (mdc::EventState)state;
}


- (void)mouseDown:(NSEvent *)theEvent
{
  NSPoint point= [self convertPoint:[theEvent locationInWindow]
                           fromView:nil];
  mdc::EventState state= makeEventState(theEvent);
  mdc::MouseButton button= (mdc::MouseButton)[theEvent buttonNumber];
  
  switch (button)
  {
    case mdc::ButtonLeft: _buttonState|= mdc::SLeftButtonMask;
    case mdc::ButtonMiddle: _buttonState|= mdc::SMiddleButtonMask;
    case mdc::ButtonRight: _buttonState|= mdc::SRightButtonMask;
  }
  
  if (![[self delegate] canvasMouseDown:button location:point state:(mdc::EventState)(state|_buttonState)])
  {  
    _view->handle_mouse_button(button, true, point.x, point.y, (mdc::EventState)(state|_buttonState));
  }
}


- (void)mouseUp:(NSEvent *)theEvent
{
  NSPoint point= [self convertPoint:[theEvent locationInWindow]
                           fromView:nil];
  mdc::EventState state= makeEventState(theEvent);
  mdc::MouseButton button= (mdc::MouseButton)[theEvent buttonNumber];

  switch (button)
  {
    case mdc::ButtonLeft: _buttonState&= ~mdc::SLeftButtonMask;
    case mdc::ButtonMiddle: _buttonState&= ~mdc::SMiddleButtonMask;
    case mdc::ButtonRight: _buttonState&= ~mdc::SRightButtonMask;
  }
  
  if (![[self delegate] canvasMouseUp:button location:point state:(mdc::EventState)(state|_buttonState)])
  {
    _view->handle_mouse_button(button, false, point.x, point.y, (mdc::EventState)(state|_buttonState));
  }
}


- (void)mouseDragged:(NSEvent *)theEvent
{
  NSPoint point= [self convertPoint:[theEvent locationInWindow]
                           fromView:nil];
  mdc::EventState state= makeEventState(theEvent);
  
  if (![[self delegate] canvasMouseMoved:point state:(mdc::EventState)(state|_buttonState)])
  {  
    _view->handle_mouse_move(point.x, point.y, (mdc::EventState)(state|_buttonState));
  }
}


- (void)mouseMoved:(NSEvent *)theEvent
{
  [self mouseDragged:theEvent];
}


@end
