//
//  GRTShellViewController.m
//  MySQLWorkbench
//
//  Created by Alfredo Kojima on 23/Sep/08.
//  Copyright 2008 Sun Microsystems Inc. All rights reserved.
//

#import "GRTShellViewController.h"
#import "MTextImageCell.h"

#include "sigobjc++.h"
#include "grt/grt_manager.h"

@implementation GRTShellViewController


static void handle_ready(const std::string &prompt, GRTShellViewController *shell)
{
  [shell nextShellCommand:[NSString stringWithUTF8String:prompt.c_str()]];
}


static void handle_output(const std::string &output, GRTShellViewController *shell)
{
  [shell write:[NSString stringWithUTF8String:output.c_str()]];
}


- (void)dealloc
{
  bec::ListModel *model= [inspectorDS listModel];
  delete model;
  [[NSNotificationCenter defaultCenter] removeObserver:self];
  [super dealloc];
}


- (NSView*)sidebar
{
  return sidebarView;
}


- (void)setupWithGRTManager:(bec::GRTManager*)grtm
{  
  [NSBundle loadNibNamed:@"GRTShellSidebar" owner:self];
  
  _grtm= grtm;
  _promptPrinted= NO;

  _grtm->get_shell()->set_ready_handler(sigc::bind(sigc::ptr_fun(handle_ready), self));
  _grtm->get_shell()->set_output_handler(sigc::bind(sigc::ptr_fun(handle_output), self));

  [shellText replaceCharactersInRange:NSMakeRange(0,[[shellText textStorage] length]) withString:@""];

  [shellCommand setFont:[NSFont fontWithName:@"Monaco" size:10.0]];

  [shellText setTextColor:[NSColor whiteColor]];
  [shellText setFont:[NSFont fontWithName:@"Monaco" size:10.0]];
  
  [[NSNotificationCenter defaultCenter] addObserver:self
                                           selector:@selector(globalsSelectionChanged:)
                                               name:NSOutlineViewSelectionDidChangeNotification
                                             object:globalsOutline];
  
  [globalsDS setTreeModel:_grtm->get_shared_value_tree("/")];
  [classesDS setTreeModel:_grtm->get_shared_structs_tree()];
  [modulesDS setTreeModel:_grtm->get_shared_modules_tree()];
}

- (void)handleGRTMessage:(const grt::Message*)msg
{
  [self writeLine: [NSString stringWithUTF8String: msg->format().c_str()]];
}


- (void)refresh
{
  _grtm->get_shared_value_tree("/")->refresh();
  _grtm->get_shared_structs_tree()->refresh();
  _grtm->get_shared_modules_tree()->refresh();
  
  [globalsOutline reloadData];
  [classesOutline reloadData];
  [modulesOutline reloadData];
}


- (void)globalsSelectionChanged:(NSNotification*)notif
{
  int row= [globalsOutline selectedRow];
  if (row >= 0)
  {
    bec::ListModel *model= [inspectorDS listModel];
    delete model;
    
    grt::ValueRef value;
    
    value= ((bec::ValueTreeBE*)[globalsDS treeModel])->get_node_value([globalsDS nodeIdForItem:[globalsOutline itemAtRow:row]]);
  
    [inspectorDS setListModel:_grtm->get_new_value_inspector(value)];
  }
  else
    [inspectorDS setListModel:0];
  [inspectorTable reloadData];
}


- (void)nextShellCommand:(NSString*)prompt
{
  float totalWidth= NSWidth([[shellPrompt superview] frame]);
  float promptWidth;
  NSRect commandFrame= [shellCommand frame];
  
  [shellPrompt setStringValue:prompt];
  [shellPrompt sizeToFit];
  
  promptWidth= NSWidth([shellPrompt frame]);

  commandFrame.size.width= totalWidth - promptWidth - 12;
  commandFrame.origin.x= totalWidth - NSWidth(commandFrame);

  [shellCommand setFrame: commandFrame];
}


- (void)writeLine:(NSString*)text
{
  if (_promptPrinted)
  {
    [shellText replaceCharactersInRange:NSMakeRange([[shellText textStorage] length], 0) withString:@"\n"];
    _promptPrinted= NO;
  }
  
  [shellText replaceCharactersInRange:NSMakeRange([[shellText textStorage] length], 0) withString:text];
  [shellText replaceCharactersInRange:NSMakeRange([[shellText textStorage] length], 0) withString:@"\n"];
  
  [shellText scrollRangeToVisible:NSMakeRange([[shellText textStorage] length], 0)];
}


- (void)write:(NSString*)text
{
  if (_promptPrinted)
  {
    [shellText replaceCharactersInRange:NSMakeRange([[shellText textStorage] length], 0) withString:@"\n"];
    _promptPrinted= NO;
  }
  
  [shellText replaceCharactersInRange:NSMakeRange([[shellText textStorage] length], 0) withString:text];
  
  [shellText scrollRangeToVisible:NSMakeRange([[shellText textStorage] length], 0)];
}


- (IBAction)activateCommand:(id)sender
{
  NSString *command= [[shellCommand stringValue] stringByAppendingString:@"\n"];

  _promptPrinted= NO;
  [self write: [NSString stringWithFormat:@"\n%@ %@", [shellPrompt stringValue], [shellCommand stringValue]]];
  _promptPrinted= YES;

  _grtm->get_shell()->process_line_async([command UTF8String]);
  
  [shellCommand setStringValue:@""];
  [[shellCommand window] makeFirstResponder: shellCommand];
}

@end
