//
//  GRTTreeDataSource.mm
//  MySQLWorkbench
//
//  Created by Alfredo Kojima on 29/Sep/08.
//  Copyright 2008 Sun Microsystems Inc. All rights reserved.
//

#import "GRTTreeDataSource.h"
#import "GRTIconCache.h"
#import "MTextImageCell.h"
 
@implementation GRTTreeDataSource

- (id)initWithTreeModel:(bec::TreeModel*)model
{
  if ((self = [super init]) != nil)
  {
    _tree= model;
  }
  return self;
}


- (void)setTreeModel:(bec::TreeModel*)model
{
  _tree= model;
}


- (bec::TreeModel*)treeModel
{
  return _tree;
}

- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item
{
  //XXX this will leak mem, maybe should cast item directly to NodeId
  return [[GRTNodeId nodeIdWithNodeId:_tree->get_child(item ? [item nodeId] : bec::NodeId(), index)] retain];
}


- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{
  return !item || _tree->is_expandable([item nodeId]);
}


- (bec::NodeId)nodeIdForItem:(id)item
{
  return [item nodeId];
}


- (void)outlineViewItemWillExpand:(NSNotification *)notification
{
  id item= [[notification userInfo] objectForKey:@"NSObject"];
  
  if (_tree)
    _tree->expand_node(item ? [item nodeId] : bec::NodeId());
}


- (void)outlineViewItemDidCollapse:(NSNotification *)notification
{
  id item= [[notification userInfo] objectForKey:@"NSObject"];
  
  if (_tree)
    _tree->collapse_node(item ? [item nodeId] : bec::NodeId());
}


- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
  if (_tree)
    return _tree->count_children(item ? [item nodeId] : bec::NodeId());
  return 0;
}


- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
  std::string strvalue;
  int column= 0;

  if (!_tree->get_field([item nodeId], column, strvalue))
    return nil;
  
  return [NSString stringWithUTF8String:strvalue.c_str()];
}



- (void)outlineView:(NSOutlineView *)outlineView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn item:(id)item
{
  if ([[tableColumn identifier] isEqualTo:@"0"])
  {
    bec::IconId icon_id= _tree->get_field_icon([item nodeId], 0, bec::Icon16);

    NSLog(@"WILLDISPLAY %s", [item nodeId].repr().c_str());
    
    if (icon_id == 0 && _tree->is_expandable([item nodeId]))
    {      
      // XXX set default folder icon here
    }

    if (icon_id != 0)
      [cell setImage:[[GRTIconCache sharedIconCache] imageForIconId:icon_id]];
  }
}


@end
