//
//  WBModelViewPanel.mm
//  MySQLWorkbench
//
//  Created by Alfredo Kojima on 6/Oct/08.
//  Copyright 2008 Sun Microsystems Inc. All rights reserved.
//

#import "sigobjc++.h"
#include "workbench/wb_model_view_form.h"

#import "WBModelViewPanel.h"
#import "MCanvasScrollView.h"
#import "GRTIconCache.h"

@implementation WBModelViewPanel

- (id)initWithId:(NSString*)oid
          formBE:(wb::ModelViewForm*)be
{
  self= [super init];
  if (self != nil)
  {
    _formBE= be;
    
    _identifier= [oid retain];
    _viewer= [[MCanvasViewer alloc] initWithFrame:NSMakeRect(0, 0, 300, 300)];
    _scrollView= [[MCanvasScrollView alloc] initWithFrame:NSMakeRect(0, 0, 100+[NSScroller scrollerWidth], 100+[NSScroller scrollerWidth])];
    
    [_viewer setupQuartz];
    [_viewer setDelegate:self];
    [_scrollView setContentCanvas:_viewer];
//    [self 
  }
  return self;
}


- (void)dealloc
{
  [_viewer release];
  [_scrollView release];
  [super dealloc];
}


- (NSView*)topView
{
  return _scrollView;
}


- (mdc::CanvasView*)canvas
{
  return [_viewer canvas];
}


- (NSString*)identifier
{
  return _identifier;
}


- (NSString*)title
{
  return [NSString stringWithUTF8String:_formBE->get_title().c_str()];
}


static NSPoint loadCursorHotspot(const std::string &path)
{
  gsize size;
  guint8 *buffer;
  
  if (g_file_get_contents(path.c_str(), (gchar**)&buffer, &size, NULL))
  {
    if (buffer[0] != 0 || buffer[1] != 0 || buffer[2] != 2 || buffer[3] != 0)
    {
      g_free(buffer);
      return NSMakePoint(0.0, 0.0);
    }
    int xspot= buffer[6+4]|buffer[6+5]<<8;
    int yspot= buffer[6+6]|buffer[6+7]<<8;
    g_free(buffer);
    return NSMakePoint(xspot, yspot);
  } 
  return NSMakePoint(0.0, 0.0);
}


- (void)updateCursor
{
  std::string cursorName= _formBE->get_cursor();
  NSCursor *cursor;
  NSImage *image= [[GRTIconCache sharedIconCache] imageForFileName:[NSString stringWithFormat:@"%s.cur", cursorName.c_str()]];
  
  NSString *path= [[NSBundle mainBundle] pathForResource:[NSString stringWithUTF8String:cursorName.c_str()]
                                                  ofType:@"cur" inDirectory:@"images"];
  
  if (!path)
    cursor= nil;
  else
    cursor= [[NSCursor alloc] initWithImage:image hotSpot:loadCursorHotspot([path fileSystemRepresentation])];
  
  [_viewer setCursor:cursor];
  [cursor release];
}


- (bec::UIForm*)formBE
{
  return _formBE;
}


- (BOOL)canvasMouseDown:(mdc::MouseButton)button
               location:(NSPoint)pos
                  state:(mdc::EventState)state
{
  _formBE->handle_mouse_button(button, true, pos.x, pos.y, state);  
  return YES;
}

- (BOOL)canvasMouseUp:(mdc::MouseButton)button
             location:(NSPoint)pos
                state:(mdc::EventState)state
{
  _formBE->handle_mouse_button(button, false, pos.x, pos.y, state);
  return YES;
}

- (BOOL)canvasMouseMoved:(NSPoint)pos
                   state:(mdc::EventState)state
{
  _formBE->handle_mouse_move(pos.x, pos.y, state);
  
  return YES;
}


//void handle_key(const mdc::KeyInfo &key, bool press, mdc::EventState state);

@end
