using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI.Docking;
using MySQL.Workbench;
using MySQL.GUI.Workbench;
using MySQL.GUI.Workbench.Properties;
using MySQL.GUI.Mdc;
using MySQL.Grt;
using Aga.Controls.Tree;
using Aga.Controls.Tree.NodeControls;
using System.Text.RegularExpressions;

namespace MySQL.GUI.Workbench
{
	public partial class ApplicationOptionsForm : DockContent
	{
		#region Member Variables

		protected WbContext wbContext;
    private String modelId= "";
		private OptionsTreeModel OptionsTreeModel;

		// Stringlist with option-keys that are specially handled (hard coded)
		private List<String> explicitOptions;

    private bool updatingFont = false;
    private Dictionary<String,String> fontOptions;
    private Dictionary<String, String> fontOptionValues;

		#endregion

		#region Constructor

		public ApplicationOptionsForm(WbContext WbContext)
		{
      wbContext = WbContext;

      InitializeComponent();


			// create explicit options list
			explicitOptions = new List<string>();

			// fill explicit controls with data and build explicit Option list for 
			// excluding them from advancedOptions-treeView
			fillControls();

      fontOptions = new Dictionary<string, string>();
      fontOptionValues = new Dictionary<string, string>();

      fontTargetList.Sorted = true;
      // fill list of font options
      fontTargetList.Items.Clear();
      List<String> options= wbContext.get_wb_options_keys("");
      Regex camelSplitter = new Regex("([A-Z][a-z]*)");
      foreach (String option in options)
      {
        if (option.EndsWith("Font") && option.StartsWith("workbench."))
        {
          String part = option.Substring(option.IndexOf(':') + 1);
          String figure = option.Substring(0, option.IndexOf(':')).Split(new char[] { '.' })[2];
          String caption = "";

          part = part.Substring(0, part.Length - 4);

          foreach (String p in camelSplitter.Split(figure))
            caption = caption + p + " ";

          caption = caption + part;

          fontTargetList.Items.Add(caption);

          fontOptions[caption]= option;
          fontOptionValues[option] = wbContext.get_wb_options_value("", option);

          // add to explicit options list
          explicitOptions.Add(option);
        }
      }


			// generate treeModel for advancedOptions treeView
			OptionsTreeModel = new OptionsTreeModel(OptionsTreeView, explicitOptions, wbContext, "", keyNodeControl, valueNodeControl);
			OptionsTreeView.Model = OptionsTreeModel;

      useGlobalCheck.Visible = false;

      // fill list of fonts
      fontCombo.Items.Clear();
      System.Drawing.Text.InstalledFontCollection fonts = new System.Drawing.Text.InstalledFontCollection();
      foreach (FontFamily family in fonts.Families)
        fontCombo.Items.Add(family.Name);

      fontTargetList.SelectedIndex = 0;
		}


    public ApplicationOptionsForm(WbContext WbContext, String ModelId)
    {
      wbContext = WbContext;
      modelId = ModelId;

      InitializeComponent();


      // create explicit options list
      explicitOptions = new List<string>();

      // fill explicit controls with data and build explicit Option list for 
      // excluding them from advancedOptions-treeView
      fillControls();

      // generate treeModel for advancedOptions treeView
      OptionsTreeModel = new OptionsTreeModel(OptionsTreeView, explicitOptions, wbContext, modelId, keyNodeControl, valueNodeControl);
      OptionsTreeView.Model = OptionsTreeModel;

      useGlobalCheck.Visible = true;

      OptionsTabControl.TabPages.Remove(generalTabPage);
      OptionsTabControl.TabPages.Remove(colorTabPage);
      OptionsTabControl.TabPages.Remove(fontTabPage);
      OptionsTabControl.TabPages.Remove(advTabPage);
    }

		#endregion


		#region FormCode

    private Control findControlWithTag(Control ctrl, String tag)
    {
      if (ctrl.Tag != null && String.Compare(ctrl.Tag.ToString(), tag) == 0 &&
          (ctrl.GetType() == typeof(TextBox) || ctrl.GetType() == typeof(ComboBox) 
            || ctrl.GetType() == typeof(MaskedTextBox)
            || ctrl.GetType() == typeof(NumericUpDown)
            || ctrl.GetType() == typeof(RadioButton)
            || ctrl.GetType() == typeof(GroupBox)
            || ctrl.GetType() == typeof(CheckBox)))
        return ctrl;

      foreach (Control c in ctrl.Controls)
      {
        Control r= findControlWithTag(c, tag);
        if (r != null)
          return r;
      }
      return null;
    }

		protected void fillControls()
		{
			List<String> keys = wbContext.get_wb_options_keys(modelId);
			foreach (String key in keys)
			{
				// find explicit control for option-key
        Control expControl;
				foreach(TabPage tpage in OptionsTabControl.TabPages)
				{
          expControl = findControlWithTag(tpage, key);

					if (expControl != null)
					{
            if (expControl.GetType() == typeof(CheckBox))
            {
              CheckBox cb = expControl as CheckBox;
              cb.Checked = int.Parse(wbContext.get_wb_options_value(modelId, key)) != 0;
            }
            else if (expControl.GetType() == typeof(NumericUpDown))
            {
              NumericUpDown updown = expControl as NumericUpDown;
              updown.Value = int.Parse(wbContext.get_wb_options_value(modelId, key));
            }
            else if (expControl.GetType() == typeof(GroupBox))
            {
              GroupBox box= expControl as GroupBox;

              String value= wbContext.get_wb_options_value(modelId, key);

              RadioButton option = findControlWithTag(box, value) as RadioButton;
              if (option != null)
                option.Checked = true;
            }
            else if (expControl.GetType() == typeof(ComboBox))
            {
              ComboBox combo = expControl as ComboBox;

              combo.Items.Clear();
              String options = wbContext.get_wb_options_value(modelId, "@"+key+"/Items");
              String selected = wbContext.get_wb_options_value(modelId, key);
              if (options != null)
                combo.Items.AddRange(options.Split(new char[] { ',' }));

              combo.SelectedItem = selected;
            }
            else
            {
              // if control found, fill in value for this option
              expControl.Text = wbContext.get_wb_options_value(modelId, key);
            }
						// add to explicit options list
						explicitOptions.Add(key);
            break;
					}
				}
			}

      if (modelId != "" && wbContext.get_wb_options_value(modelId, "useglobal") == "1")
        useGlobalCheck.Checked = true;
      else
        useGlobalCheck.Checked = false;
		}


    protected void commitControls()
    {
      if (modelId != "" && useGlobalCheck.Checked)
      {
        wbContext.discard_wb_model_options(modelId);
        return;
      }

      List<String> keys = wbContext.get_wb_options_keys(modelId);

      foreach (String key in keys)
      {
        // find explicit control for option-key
        Control expControl;
        foreach (TabPage tpage in OptionsTabControl.TabPages)
        {
          expControl = findControlWithTag(tpage, key);

          if (expControl != null)
          {
            if (expControl.GetType() == typeof(NumericUpDown))
            {
              NumericUpDown updown = expControl as NumericUpDown;
              wbContext.set_wb_options_value(modelId, key, updown.Value.ToString());
            }
            else if (expControl.GetType() == typeof(CheckBox))
            {
              CheckBox check = expControl as CheckBox;
              wbContext.set_wb_options_value(modelId, key, check.Checked ? "1" : "0");
            }
            else if (expControl.GetType() == typeof(GroupBox))
            {
              GroupBox box = expControl as GroupBox;
              String value= null;
              foreach (Control c in box.Controls)
              {
                RadioButton radio = c as RadioButton;
                if (radio.Checked)
                {
                  value = radio.Tag as String;
                  break;
                }
              }
              if (value != null)
                wbContext.set_wb_options_value(modelId, key, value);
            }
            else
              wbContext.set_wb_options_value(modelId, key, expControl.Text);
            break;
          }
        }
      }
    }


		private void okButton_Click(object sender, EventArgs e)
		{
      wbContext.begin_wb_options_change();
      commitControls();
      OptionsTreeModel.StoreValues();
      if (modelId == "")
        saveFontOptions();
      wbContext.end_wb_options_change();
		}

		private void cancelButton_Click(object sender, EventArgs e)
		{
		}

		private void colorTextBox_Validating(object sender, CancelEventArgs e)
		{
			Regex validator = new Regex(@"^\#[0-9a-fA-F]{6}$");

			e.Cancel = !validator.IsMatch(((Control)sender).Text);
		}

    private void useGlobalCheck_CheckedChanged(object sender, EventArgs e)
    {
      OptionsTabControl.Enabled = !useGlobalCheck.Checked;
    }

    private void fontTargetList_SelectedIndexChanged(object sender, EventArgs e)
    {
      if (fontTargetList.SelectedIndex >= 0)
        parseFontString(fontOptionValues[fontOptions[fontTargetList.SelectedItem.ToString()]]);
      updateSample();
    }

    private void fontCombo_SelectedIndexChanged(object sender, EventArgs e)
    {
      if (!updatingFont)
      {
        if (fontTargetList.SelectedIndex >= 0)
          fontOptionValues[fontOptions[fontTargetList.SelectedItem.ToString()]] = updateUIAndReturnFontString();
        updateSample();
      }
    }

    private void fontSizeUpDown_ValueChanged(object sender, EventArgs e)
    {
      if (!updatingFont)
      {
        if (fontTargetList.SelectedIndex >= 0)
          fontOptionValues[fontOptions[fontTargetList.SelectedItem.ToString()]] = updateUIAndReturnFontString();
        updateSample();
      }
    }

    private void boldCheckBox_CheckedChanged(object sender, EventArgs e)
    {
      if (!updatingFont)
      {
        if (fontTargetList.SelectedIndex >= 0)
          fontOptionValues[fontOptions[fontTargetList.SelectedItem.ToString()]] = updateUIAndReturnFontString();
        updateSample();
      }
    }

    private void italicCheckBox_CheckedChanged(object sender, EventArgs e)
    {
      if (!updatingFont)
      {
        if (fontTargetList.SelectedIndex >= 0)
          fontOptionValues[fontOptions[fontTargetList.SelectedItem.ToString()]] = updateUIAndReturnFontString();
        updateSample();
      }
    }

    private void saveFontOptions()
    {
      foreach(String option in fontOptionValues.Keys)
      {
        wbContext.set_wb_options_value("", option, fontOptionValues[option]);
      }
    }

    #endregion

    String updateUIAndReturnFontString()
    {
      // Not all fonts support all font styles so we have to make sure it is impossible
      // to select invalid styles.
      String formatString = fontCombo.SelectedItem.ToString();
      updateCheckboxesAccordingToValidStyles(formatString);

      String size = fontSizeUpDown.Value.ToString();

      if (boldCheckBox.Checked)
        formatString += " Bold";
      if (italicCheckBox.Checked)
        formatString += " Italic";
      formatString += " " + size;

      return formatString;
    }

    bool parseFontString(String font)
    {
      String[] parts= font.Split(new char[] { ' ' });

      int index = parts.Length - 1;

      updatingFont = true;

      fontSizeUpDown.Value = 12;
      if (index > 0)
      {
        try
        {
          fontSizeUpDown.Value= int.Parse(parts[index]);
          --index;
        }
        catch
        {
          // Swallow any conversion error.
        }
      }

      boldCheckBox.Checked = false;
      italicCheckBox.Checked = false;
      for (int i = 0; i < 2; i++)
      {
        if (index > 0)
        {
          if (parts[index].ToLower() == "bold")
          {
            boldCheckBox.Checked = true;
            --index;
          }
          else 
            if (parts[index].ToLower() == "italic")
            {
              italicCheckBox.Checked = true;
              --index;
            }
        }
      }

      String familyName = parts[0];
      for (int i = 1; i <= index; i++)
        familyName += " " + parts[i];

      // Make sure we have a valid combination of font family and font styles.
      // If only certain styles are not supported correct the checkboxes. If the font family
      // is wrong (or none of the needed styles is supported) default to Arial.
      bool validFont = (fontCombo.Items.IndexOf(familyName) >= 0) &&
        updateCheckboxesAccordingToValidStyles(familyName);
      if (validFont)
        fontCombo.SelectedItem = familyName;
      else
        fontCombo.SelectedItem = "Arial";

      updatingFont = false;

      return true;
    }

    /**
     * Returns true if at least one valid style could be found.
     */
    private bool updateCheckboxesAccordingToValidStyles(String familyName)
    {
      FontFamily family = new FontFamily(familyName);
      bool validFont = family.IsStyleAvailable(FontStyle.Regular) || family.IsStyleAvailable(FontStyle.Bold) ||
        family.IsStyleAvailable(FontStyle.Italic);

      if (validFont)
      {
        // Adjust check boxes if a style is not available.
        if (boldCheckBox.Checked && !family.IsStyleAvailable(FontStyle.Bold))
          boldCheckBox.Checked = false;
        if (italicCheckBox.Checked && !family.IsStyleAvailable(FontStyle.Italic))
          italicCheckBox.Checked = false;

        // If the regular style is not available then select one of the other styles.
        // The above check ensures we have at least one available. Prefer bold over italic.
        if (!family.IsStyleAvailable(FontStyle.Regular))
          if (family.IsStyleAvailable(FontStyle.Bold))
            boldCheckBox.Checked = true;
          else
            italicCheckBox.Checked = true;
      }
      return validFont;
    }

    void updateSample()
    {
      FontStyle style = FontStyle.Regular;
      if (boldCheckBox.Checked)
        style |= FontStyle.Bold;
      if (italicCheckBox.Checked)
        style |= FontStyle.Italic;

      sampleLabel.Font = new Font((String)fontCombo.SelectedItem, (float)fontSizeUpDown.Value, style);
    }

  }
}