#ifndef _LF_RADIOBUTTON_H_
#define _LF_RADIOBUTTON_H_

#include <mforms/radiobutton.h>


#include "lf_view.h"
#include "lf_box.h"

namespace mforms {
namespace gtk {

class RadioButtonImpl : public ViewImpl
{
 protected:
  Gtk::RadioButton *_radio;

  virtual Gtk::Widget *get_outer() const { return _radio; }

  RadioButtonImpl(::mforms::RadioButton *self)
    : ViewImpl(self)
  {
    _radio= Gtk::manage(new Gtk::RadioButton());

    _radio->signal_parent_changed().connect(sigc::mem_fun(this, &RadioButtonImpl::parent_changed));
    _radio->signal_toggled().connect(sigc::mem_fun(*self, &::mforms::RadioButton::callback));
  }

  static bool create(::mforms::RadioButton *self)
  {
    return new RadioButtonImpl(self);
  }

  static void set_text(::mforms::RadioButton *self, const std::string &text)
  {
    if ( self )
    {
      RadioButtonImpl* button = self->get_data<RadioButtonImpl>();

      if ( button )
      {
        button->_radio->set_label(text);
      }
    }
  }
  
  static bool get_active(::mforms::RadioButton *self)
  {
    RadioButtonImpl* button = self->get_data<RadioButtonImpl>();
    
    if ( button )
    {
      return button->_radio->get_active();
    }
    return false;
  }
  
  
  static void set_active(::mforms::RadioButton *self, bool flag)
  {
    RadioButtonImpl* button = self->get_data<RadioButtonImpl>();
    
    if ( button )
    {
      button->_radio->set_active(flag);
    }
  }


  void parent_changed(Gtk::Widget *old_parent)
  {
    Box *box= dynamic_cast<Box*>(((View*)owner)->get_parent());

    if (box)
    {
      box->get_data<BoxImpl>()->add_to_radio_group(_radio);
    }
  }

public:
  static void init()
  {
    ::mforms::ControlFactory *f = ::mforms::ControlFactory::get_instance();

    f->_radio_impl.create= &RadioButtonImpl::create;
    f->_radio_impl.set_text= &RadioButtonImpl::set_text;
    f->_radio_impl.get_active= &RadioButtonImpl::get_active;
    f->_radio_impl.set_active= &RadioButtonImpl::set_active;
  }
};

}
}



#endif /* _LF_RADIOBUTTON_H_ */
