#ifndef _WF_FORM_H_
#define _WF_FORM_H_

#include <mforms/mforms.h>


#include "wf_button.h"
#include "wf_view.h"
#include "wf_mforms.h"

using namespace System;
using namespace Windows::Forms;

#pragma make_public(::mforms::Form)

namespace MySQL {
  namespace Forms {

    public ref class FormImpl : public ViewImpl
    {
      static bool create(::mforms::Form *self, ::mforms::Form *owner)
      {
        FormImpl ^form= gcnew FormImpl(self, owner);

        if (form != nullptr)
        {
          Windows::Forms::Form ^f= gcnew Windows::Forms::Form();
          form->wcontrol= f;
          form->wcontrol->Margin.All= 0;
          form->wcontrol->Padding.All= 0;
          f->StartPosition= FormStartPosition::CenterScreen;
          return true;
        }
        return false;
      }

      static void set_title(::mforms::Form *self, const std::string &title)
      {
        FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);

        if (form != nullptr)
        {
          ((Form^)form->wcontrol)->Text= gcnew String(title.c_str());
        }
      }

      static void show_modal(::mforms::Form *self, ::mforms::Button *accept,::mforms::Button *cancel)
      {
        FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);

        if (form != nullptr)
        {
          Form ^f= (Form^)form->wcontrol;

          if (accept)
          {
            f->AcceptButton= (Button^)((ViewImpl^)ObjectImpl::FromUnmanaged(accept))->get_control();
            f->AcceptButton->DialogResult= DialogResult::OK;
          }
          if (cancel)
          {
            f->CancelButton= (Button^)((ViewImpl^)ObjectImpl::FromUnmanaged(cancel))->get_control();
            f->CancelButton->DialogResult= DialogResult::Cancel;
          }
          // XXX show modal should show the window but not block
          f->Show();
          //return f->ShowDialog() == DialogResult::OK;
        }
        //return false;
      }

      static bool run_modal(::mforms::Form *self, ::mforms::Button *accept,::mforms::Button *cancel)
      {
        FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);

        if (form != nullptr)
        {
          Form ^f= (Form^)form->wcontrol;

          if (accept)
          {
            f->AcceptButton= (Button^)((ViewImpl^)ObjectImpl::FromUnmanaged(accept))->get_control();
            f->AcceptButton->DialogResult= DialogResult::OK;
          }
          if (cancel)
          {
            f->CancelButton= (Button^)((ViewImpl^)ObjectImpl::FromUnmanaged(cancel))->get_control();
            f->CancelButton->DialogResult= DialogResult::Cancel;
          }
          return f->ShowDialog() == DialogResult::OK;
        }
        return false;
      }

      static void close(::mforms::Form *self)
      {
        FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);

        if (form != nullptr)
        {
          ((Form^)form->wcontrol)->Close();
        }
      }


      static void set_content(::mforms::Form *self, ::mforms::View *view)
      {
        FormImpl^ form= (FormImpl^)ObjectImpl::FromUnmanaged(self);

        if (form != nullptr)
        {
          ViewImpl^ child= (ViewImpl^)ObjectImpl::FromUnmanaged(view);
          Windows::Forms::Control^ ctl= child->get_control();

          ctl->Dock= DockStyle::Fill;

          ((Form^)form->wcontrol)->Controls->Add(ctl);
        }
      }

      FormImpl(::mforms::Form *form, ::mforms::Form *owner)
        : ViewImpl(form)
      {
      }


    public:
      virtual void set_size(int width, int height) override
      {
        if (width > 0)
          wcontrol->Width= width + (wcontrol->Width - wcontrol->ClientSize.Width);

        if (height > 0)
          wcontrol->Height= height + (wcontrol->Height - wcontrol->ClientSize.Height);
      }

      static void init(Manager ^mgr)
      {
        ::mforms::ControlFactory *f= ::mforms::ControlFactory::get_instance();

        DEF_CALLBACK2(bool, ::mforms::Form*, ::mforms::Form*, mgr, f->_form_impl, FormImpl, create);
        DEF_CALLBACK1(void, ::mforms::Form*, mgr, f->_form_impl, FormImpl, close);
        DEF_CALLBACK2(void, ::mforms::Form*, ::mforms::View*, mgr, f->_form_impl, FormImpl, set_content);
        DEF_CALLBACK2(void, ::mforms::Form*, const std::string&, mgr, f->_form_impl, FormImpl, set_title);
        DEF_CALLBACK3(bool, ::mforms::Form*, ::mforms::Button*, ::mforms::Button*, mgr, f->_form_impl, FormImpl, run_modal);
        DEF_CALLBACK3(void, ::mforms::Form*, ::mforms::Button*, ::mforms::Button*, mgr, f->_form_impl, FormImpl, show_modal);
      }
    };

  };
};

#endif
