--
-- Workbench Tools Plugins
--


--
-- standard module/plugin functions
-- 

function getModuleInfo()
	return {
		name= "WbTools",
		author= "MySQL AB.",
		version= "1.0",
		implements= "PluginInterface",
		functions= {
		"getPluginInfo:l<o@app.Plugin>:",
		"quickTables:i:o@db.Catalog",
        "refactorMergeTables:i:l<o@db.Table>"
		}
	}
end


-- helper function to create a descriptor for an argument of a specific type of object
function objectPluginInput(type)
	return grtV.newObj("app.PluginObjectInput", {objectStructName= type})
end

function selectionPluginInput(types, card)
	return grtV.newObj("app.PluginSelectionInput", {objectStructNames= types, argumentCardinality=card})
end


function makePlugin(name, caption, moduleName, functionName, inputValues, groups)
    local plugin

    plugin= grtV.newObj("app.Plugin", {
		name= name,
        caption= caption,
		moduleName= moduleName,
		pluginType= "normal", 
		moduleFunctionName= functionName,
		inputValues= inputValues,
		rating= 100, 
		showProgress= 0,
		groups= groups
	})
    -- fixup owner
    for i=1,grtV.getn(plugin.inputValues) do
        plugin.inputValues[i].owner= plugin
    end

    return plugin
end


function getPluginInfo()
	local l
    local plugin

    -- create the list of plugins that this module exports
	l= grtV.newList("object", "app.Plugin")

    plugin= makePlugin("wb.tools.quickTables", "Create Multiple Tables", "WbTools", "quickTables", 
        {objectPluginInput("db.Catalog")}, {"Catalog/Utilities","Menu/Objects"})
    grtV.insert(l, plugin)

    plugin= makePlugin("wb.tools.refactorMergeTables", "Merge Tables", "WbTools", "refactorMergeTables", 
        {selectionPluginInput({"db.Table"},"+")}, {"Catalog/Utilities","Menu/Objects"})
    grtV.insert(l, plugin)

	return l
end



function strsplit(str, sep)
    local parts= {}
    local pos

    pos = string.find(str, sep)
    while pos ~= nil do
        local s

        s= string.sub(str, 1, pos-1)
        str= string.sub(str, pos+1)
        table.insert(parts, s)

        pos = string.find(str, sep)
    end
    if str ~= "" then
        table.insert(parts, str)
    end

    return parts
end

function strtrim(str)
    local s,e

    s=1
    e=string.len(str)
    while str[s]==" " or str[s]=="\t" do
        s=s+1
    end
    while str[e]==" " or str[e]=="\t" do
        e=e-1
    end
    return string.sub(str,s,e)
end


function _parse_table_definitions(code)
    local defs= {}
    local tables
    local tbl
    local t,c
    local s,e

    tables= strsplit(code, "\n")
    
    for i,t in ipairs(tables) do
        t= strtrim(t)
        s= string.find(t, "[(]")
        if s == nil then
            tbl={t}
        else
            local columns
            
            e= string.find(t,"[)]")

            tbl={strtrim(string.sub(t, 1, s-1))}
            if e ~= nil then
                columns= string.sub(t,s+1,e-1)
            else
                columns= string.sub(t,s+1)
            end
            columns= strsplit(columns, ",") 
            for i,c in ipairs(columns) do
                c= strtrim(c)
                table.insert(tbl, c)
            end
        end
        table.insert(defs, tbl)
    end

    return defs
end


function _create_table(name,columns)
    local tbl
    local column
    local cdef

    tbl= grtV.newObj("db.mysql.Table")
    tbl.name= name
    
    for i,cdef in ipairs(columns) do
        column= grtV.newObj("db.mysql.Column", {owner=table, name=cdef})
        grtV.insert(tbl.columns, column)
    end

    return tbl
end


--    
-- implementation
--

function quickTables(catalog)
    local values
    local res
    local form
    local code
    local schema= catalog.schemata[1]

    form="label;You can create multiple tables by giving their outline in the following format:\\ntable1(column1,column2,column3)\\ntable2(column1,column2)\\ntable3\\netc...\n"
    form= form.."textarea;code;;5;\n"
    values= grtV.newDict()
    res= Forms:show_simple_form("Create Multiple Tables", form, values)

    if res == 1 then
        local tabledefs
        local def

        code= values["code"]

        tabledefs= _parse_table_definitions(code)

        for i,def in ipairs(tabledefs) do
            local tbl
            local name= def[1]
            local columns= def
            if name ~= nil then
                table.remove(columns, 1)
                tbl= _create_table(name, columns)
                if tbl ~= nil then
                    tbl.owner= schema
                    grtV.insert(schema.tables, tbl)
                end
            end
        end
        return 1
    end

    return 0
end


function refactorMergeTables(selection)
    if grtV.getn(selection) ~= 2 then
        form="label;Please select 2 tables to be merged."
        forms:show_simple_form("Merge Tables", form, nil)
        return 0
    end

    return 0
end


function refactorSplitTable()
    
end


function refactorColumnToLookupTable()

end



