#include "mysql_table_editor_fe.h"
#include "linux_utilities/MGGladeXML.h"
#include "grtdb/db_object_helpers.h"
#include "treemodel_wrapper.h"

#include "mysql_table_editor_insert_page.h"

//------------------------------------------------------------------------------
DbMySQLTableEditorInsertPage::DbMySQLTableEditorInsertPage(DbMySQLTableEditor *owner
                                                      ,MySQLTableEditorBE *be
                                                      ,MGGladeXML         *xml)
                             : _owner(owner)
                             , _be(be)
                             , _xml(xml)
{
  Gtk::Button *btn(0);
  _xml->get("open_editor_button", &btn);
  btn->signal_clicked().connect(sigc::mem_fun(this, &DbMySQLTableEditorInsertPage::open_editor));

  _xml->get("insert_editor_window", &_editor_window);
  _xml->get("inserts_grid", &_inserts_tv);
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorInsertPage::refresh()
{
  Gtk::TextView *tv(0);
  _xml->get("inserts_textview", &tv);
  tv->get_buffer()->set_text(_be->get_inserts());
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorInsertPage::open_editor()
{
  update_editor();
  _editor_window->show();
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorInsertPage::update_editor()
{
  ::bec::TableInsertsGridBE *grid = _be->get_inserts_grid();
    
  _inserts_tv->remove_all_columns();  
  _inserts_tv->unset_model();

  Glib::RefPtr<ListModelWrapper> new_model = ListModelWrapper::create(grid, _inserts_tv);

  const int n_cols = grid->get_column_count();
  for ( int i = 0; i < n_cols; ++i )
  {
    switch ( grid->get_column_type(i) )
    {
      case ::bec::GridModel::StringType:
      case ::bec::GridModel::DatetimeType:
      case ::bec::GridModel::BlobType: 
      {
        new_model->model().append_string_column(i, grid->get_column_caption(i), EDITABLE, NO_ICON);
        break;    
      }
      case ::bec::GridModel::NumericType:
      {
        new_model->model().append_int_column(i, grid->get_column_caption(i), EDITABLE);
        break;
      }
    }
  }
  
  _inserts_model = new_model;
  _inserts_tv->set_model(_inserts_model);
}

