#include "mysql_table_editor_fe.h"
#include "linux_utilities/MGGladeXML.h"
#include "grtdb/db_object_helpers.h"

#include "mysql_table_editor_trigger_page.h"

//------------------------------------------------------------------------------
DbMySQLTableEditorTriggerPage::DbMySQLTableEditorTriggerPage(DbMySQLTableEditor *owner
                                                          ,MySQLTableEditorBE *be
                                                          ,MGGladeXML         *xml)
                             : _owner(owner)
                             , _be(be)
                             , _xml(xml)
                             , _errors_model(Gtk::ListStore::create(_errors_columns))
                             , _pane_position(0)
{
  Gtk::VBox* trigger_code_win;
  _xml->get("trigger_code_holder", &trigger_code_win);
  _trigger_code.widget().set_size_request(-1, 100);
  trigger_code_win->add(_trigger_code.widget());
  _owner->add_sqleditor_text_change_timer(&_trigger_code, sigc::mem_fun(this, &DbMySQLTableEditorTriggerPage::parse_sql));
  trigger_code_win->resize_children();
  
  _xml->get("trigger_parsing_errors", &_errors_tv);
  _errors_tv->set_model(_errors_model);
  
  _xml->get("trigger_paned", &_paned);
  _pane_position = _paned->property_max_position();
  _paned->set_position(_pane_position);
  _pane_pos_changed_conn = _paned->property_position().signal_changed().connect(sigc::mem_fun(this, &DbMySQLTableEditorTriggerPage::track_pane));

  _errors_tv->append_column("Line"   , _errors_columns.lineno);
  _errors_tv->append_column("Message", _errors_columns.msg);
  _errors_tv->signal_cursor_changed().connect(sigc::mem_fun(*this, &DbMySQLTableEditorTriggerPage::error_selected));

  _trigger_code.set_text(_be->get_all_triggers_sql());
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorTriggerPage::refresh()
{
}

//------------------------------------------------------------------------------
bool DbMySQLTableEditorTriggerPage::process_event(GdkEvent* event)
{
  return false;
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorTriggerPage::parse_sql(const std::string& sql)
{
  const int max_position = _paned->property_max_position();
  _pane_pos_changed_conn.block();
  _paned->set_position(max_position);
  _pane_pos_changed_conn.unblock();

  _errors_model->clear();
  _be->set_sql_parser_err_cb(sigc::mem_fun(this, &DbMySQLTableEditorTriggerPage::editor_parse_error_cb));
  _be->set_sql_parser_log_cb(sigc::mem_fun(this, &DbMySQLTableEditorTriggerPage::editor_parse_log_cb));
  _be->set_triggers_sql(_trigger_code.get_text());
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorTriggerPage::editor_parse_error_cb(const int lineno, const std::string& errormsg)
{
  Gtk::TreeModel::Row row = *(_errors_model->append());
  std::string msg(errormsg);
  std::replace(msg.begin(), msg.end(), '\n', ' ');
  row[_errors_columns.lineno]    = lineno;
  row[_errors_columns.msg]       = msg;
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorTriggerPage::editor_parse_log_cb(const std::vector<std::string>& list)
{
  const int size = list.size();
  if ( size > 0 )
  {
    _pane_pos_changed_conn.block();
    if ( !_pane_position )
    {
      Gtk::Paned* pane(0);
      _xml->get("trigger_paned", &pane);
      int size = pane->get_height();
      _pane_position = size - 80;
    }
    _paned->set_position(_pane_position);
    _pane_pos_changed_conn.unblock();
  }
  
  for ( int i = 0; i < size; ++i )
  {
    g_message("\tLOG: %s", list[i].c_str());
  }
  g_message("==============================");  
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorTriggerPage::error_selected()
{
  Gtk::TreeModel::Path   path;
  Gtk::TreeView::Column *column(0);
  
  _errors_tv->get_cursor(path, column);

  Gtk::TreeModel::Row row = *(_errors_model->get_iter(path));

  const int lineno = row[_errors_columns.lineno];

  _trigger_code.scroll_to(lineno);
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorTriggerPage::track_pane()
{
  _pane_position = _paned->get_position();
}
