#ifndef __WB_MYSQL_VIEW_EDITOR_H__
#define __WB_MYSQL_VIEW_EDITOR_H__

#include "wb_config.h"
#include "linux_utilities/plugin_editor_base.h"
#include "../backend/mysql_view_editor.h"
#include "linux_utilities/MGGladeXML.h"
#include "grtdb/db_object_helpers.h"
#include "treemodel_wrapper.h"
#include "sql_editor_fe.h"

class MGGladeXML;

//==============================================================================
//
//==============================================================================
class DbMySQLViewEditor : public PluginEditorBase
{
  MySQLViewEditorBE                 _be;
  MGGladeXML                        *_xml;
  
  virtual bec::BaseEditor *get_be();
  
 public:
  DbMySQLViewEditor(bec::GRTManager *grtm, const grt::BaseListRef &args);
  
  virtual ~DbMySQLViewEditor();
  virtual void do_refresh_form_data();
  virtual std::string get_title();
  
  SqlEditorFE           _sql_editor;
};

DbMySQLViewEditor::DbMySQLViewEditor(bec::GRTManager *grtm, const grt::BaseListRef &args)
    : PluginEditorBase(grtm, args), _be(grtm, db_mysql_ViewRef::cast_from(args[0]), get_rdbms_for_db_object(args[0]))
{
  _xml= new MGGladeXML(grtm->get_data_file_path("modules/data/editor_view.glade"));

  Gtk::Notebook *editor_window(0);
  _xml->get("mysql_view_editor_notebook", &editor_window);

  Gtk::Image *image;
  _xml->get("view_editor_image", &image);
  image->set(ImageCache::get_instance()->image_from_filename("db.View.editor.48x48.png", false));
  _xml->get("view_editor_image2", &image);
  image->set(ImageCache::get_instance()->image_from_filename("db.View.editor.48x48.png", false));

  _be.set_refresh_ui_slot(sigc::mem_fun(this, &DbMySQLViewEditor::refresh_form_data));
  
  editor_window->reparent(*this);
  editor_window->show();

  Gtk::TextView *tview(0);
  _xml->get("viewcomment", &tview);
  add_text_change_timer(tview, sigc::mem_fun(_be, &MySQLViewEditorBE::set_comment));

  Gtk::VBox *ddl_win;
  _xml->get("view_ddl", &ddl_win);
  _sql_editor.widget().set_size_request(-1, 100);
  ddl_win->add(_sql_editor.widget());
  add_sqleditor_text_change_timer(&_sql_editor, sigc::mem_fun(_be, &MySQLViewEditorBE::set_query));
  ddl_win->resize_children();

  
  refresh_form_data();
}

//------------------------------------------------------------------------------
DbMySQLViewEditor::~DbMySQLViewEditor()
{
  delete _xml;
}

//------------------------------------------------------------------------------
std::string DbMySQLViewEditor::get_title()
{
  return _be.get_name();
}

//------------------------------------------------------------------------------
bec::BaseEditor *DbMySQLViewEditor::get_be()
{
  return &_be;
}

//------------------------------------------------------------------------------
void DbMySQLViewEditor::do_refresh_form_data()
{
  Gtk::Entry* entry(0);
  _xml->get("view_name", &entry);
  entry->set_text(_be.get_name());

  Gtk::TextView *tview(0);
  _xml->get("viewcomment", &tview);
  tview->get_buffer()->set_text(_be.get_comment());
  
  std::string query(_be.get_query());
  if ( std::string::npos == _sql_editor.get_text().find(query, 0) )
    _sql_editor.set_text(query);

}

//------------------------------------------------------------------------------
extern "C" 
{
  GUIPluginBase *createDbMysqlViewEditor(bec::GRTManager *grtm, const grt::BaseListRef &args)
  {
    return Gtk::manage(new DbMySQLViewEditor(grtm, args));
  }
};

#endif
