using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using MySQL.Grt;
using MySQL.Utilities;
using MySQL.GUI.Workbench.Plugins;
using MySQL.GUI.Workbench.Plugins.Db;
using MySQL.GUI.Workbench.Plugins.DbMySQLScriptSync;

namespace MySQL.GUI.Workbench.Plugins.DbMySQLSync
{
  using System.Collections;
  using Pages = LinkedList<PageInfo>;
  using Db_object_type = MySQL.GUI.Workbench.Plugins.Db.Db_plugin.Db_object_type;
  using DbMySQLScriptSync = MySQL.GUI.Workbench.Plugins.DbMySQLScriptSync.DbMySQLScriptSync;
  using DiffTreeControl = MySQL.GUI.Workbench.Plugins.Sync.DiffTreeControl;
  using ValidationControl = MySQL.GUI.Workbench.Plugins.Sync.ValidationControl;

  public partial class SQLSyncWizardPages : WizardPluginPages
  {
    private DbMySQLSync pluginBE;

    private DbMySQLScriptSync basePluginBE;

    // dbms connection
    private DbConnection dbConnection;
    private bool showAdvancedConnectionLogPage;

    // schemata selection
    private int schema_large_image_index;
    private int schema_small_image_index;

    private List<String> schemataNames;

    // db objects filter
    private MasterSchemaObjectFilterSet master_schema_obj_filter_set;
    private Dictionary<Db_object_type, SchemaObjectFilter> dbObjectFilters;
    private SchemaObjectFilter user_filter;
    private SchemaObjectFilter table_filter;
    private SchemaObjectFilter view_filter;
    private SchemaObjectFilter routine_filter;
    private SchemaObjectFilter trigger_filter;

    private bool showAdvancedSchemataObjectsExtractionLogPage;

    // diff tree
    private DiffTreeControl diffTreeControl = null;

    // sync script execution
    private bool showAdvancedExecScriptLogPage;
    private bool syncScriptSucceeded;

    private List<String> messages;

    public SQLSyncWizardPages(WizardPlugin wizard)
      : base(wizard)
    {
      InitializeComponent();

      basePluginBE = new DbMySQLScriptSync(wizard.GrtManager);
      pluginBE = new DbMySQLSync(wizard.GrtManager);

      // dbms connection
      dbConnection = new DbConnection();
      dbConnection.Parent = connectionPanel;
      dbConnection.Dock = DockStyle.Fill;
      dbConnection.Padding = new System.Windows.Forms.Padding(5);
      dbConnection.Margin = new System.Windows.Forms.Padding(5);

      showAdvancedConnectionLogPage = false;

      // schemata selection/objects filter
      IconSize icon_size = IconSize.Icon32;
      schema_large_image_index = GrtIconManager.get_instance().add_icon_to_imagelist(pluginBE.schema_icon_id(icon_size));
      icon_size = IconSize.Icon16;
      schema_small_image_index = GrtIconManager.get_instance().add_icon_to_imagelist(pluginBE.schema_icon_id(icon_size));

      schemataListView.SmallImageList = GrtIconManager.ImageList16;
      schemataListView.LargeImageList = GrtIconManager.ImageList32;

      schemataNames = new List<String>();
      dbObjectFilters = new Dictionary<Db_object_type, SchemaObjectFilter>();

      showAdvancedSchemataObjectsExtractionLogPage = false;

      // sync script execution
      showAdvancedExecScriptLogPage = false;

      messages = new List<String>();

      AddPage(connectionPanel,
        InitializeConnectionPage,
        null,
        ProcessConnectionPage,
        null,
        null,
        "Connect to DBMS",
        "Set parameters for connection to DBMS.");

      AddPage(connectionLogPanel,
        InitializeConnectionLogPage,
        CleanupConnectionLogPage,
        ProcessConnectionLogPage,
        ToggleAdvancedConnectionLogPage,
        null,
        "Connect to DBMS (Progress/Results)",
        "See the progress/results of establishing connection to DBMS.");

      AddPage(filterSchemataPanel,
        InitializeFilterSchemataPage,
        CleanupFilterSchemataPage,
        ProcessFilterSchemataPage,
        null,
        null,
        "Select Schemata",
        "Choose schemata containing objects of interest.");

      AddPage(schemataObjectsExtractionLogPanel,
        InitializeSchemataObjectsExtractionLogPage,
        CleanupSchemataObjectsExtractionLogPage,
        ProcessSchemataObjectsExtractionLogPage,
        ToggleAdvancedSchemataObjectsExtractionLogPage,
        null,
        "Select Schemata (Progress/Results)",
        "See the progress/results of retrieving objects description from selected schemata.");

      AddPage(filterPanel,
        InitFilterPage,
        CleanupFilterPage,
        ProcessFilterPage,
        null,
        IsFilterToBeSkipped,
        "Select Objects",
        "Choose objects to synchronize.");


      AddPage(treePanel,
        InitTreePage,
        CleanupTreePage,
        ProcessTreePage,
        null,
        null,
        "SQL Diff Tree",
        "Review the diff tree and choose application direction");

      AddPage(generatedScriptPanel,
        null,
        null,
        ProcessGeneratedScriptPage,
        null,
        null,
        "SQL Sync Script",
        "Review the sync script");

      AddPage(execScriptLogPanel,
        InitializeExecScriptLogPage,
        CleanupExecScriptLogPage,
        ProcessExecScriptLogPage,
        ToggleAdvancedExecScriptLogPage,
        null,
        "Execute Generated Script (Progress/Results)",
        "See the progress/results of script execution.");
    }

#region ConnectionPage

    private string InitializeConnectionPage()
    {
      if (dbConnection.Backend == null)
      {
        connectionPanel.SuspendLayout();
        dbConnection.Backend = pluginBE.db_conn();
        connectionPanel.ResumeLayout();
      }

      return "";
    }

    private string ProcessConnectionPage()
    {
      return dbConnection.Backend.validate_driver_params();
    }
#endregion ConnectionPage

#region ConnectionLogPage

    private string InitializeConnectionLogPage()
    {
      List<String> messages = new List<String>();
      string errMsg = "";

      taskLblConnToSrcDb.TaskState = MySQLTaskStatusLabelState.TaskOpen;
      taskLblRetrSchemaInfo.TaskState = MySQLTaskStatusLabelState.TaskOpen;
      StepProlog(lConnResult, lConnStage);

      TaskMediator(connectionLogPanel, connectionLogTextBox, messages);

      // first task
      {
        lConnStage.Text = "Connecting to DBMS...";
        connectionLogPanel.Update();

        try
        {
          pluginBE.db_conn().test_connection();
          taskLblConnToSrcDb.TaskState = MySQLTaskStatusLabelState.TaskCompleted;
        }
        catch(Exception)
        {
          taskLblConnToSrcDb.TaskState = MySQLTaskStatusLabelState.TaskError;
          errMsg = "Failed to connect database.";
        }
        messages.Add(errMsg);
      }

      TaskMediator(connectionLogPanel, connectionLogTextBox, messages);

      // next task
      if (taskLblConnToSrcDb.TaskState == MySQLTaskStatusLabelState.TaskCompleted)
      {
        lConnStage.Text = "Retrieving schemata description from DBMS...";
        connectionLogPanel.Update();

        try
        {
          pluginBE.load_schemata(ref schemataNames);
          for (int n = 0; n < schemataNames.Count; ++n)
            schemataListView.Items.Add(schemataNames[n]).ImageIndex = schema_large_image_index;
          taskLblRetrSchemaInfo.TaskState = MySQLTaskStatusLabelState.TaskCompleted;
        }
        catch (Exception)
        {
          errMsg = "Failed to load schemata.";
          taskLblRetrSchemaInfo.TaskState = MySQLTaskStatusLabelState.TaskError;
        }

        messages.Add(errMsg);
      }
      else
        taskLblRetrSchemaInfo.TaskState = MySQLTaskStatusLabelState.TaskError;

      TaskMediator(connectionLogPanel, connectionLogTextBox, messages);

      StepEpilog(errMsg, lConnResult, lConnStage);

      return errMsg.Length > 0 ? "-" : "";
    }

    private string CleanupConnectionLogPage()
    {
      connectionLogTextBox.Text = "";
      schemataListView.Items.Clear();
      return "";
    }

    private string ProcessConnectionLogPage()
    {
      return "";
    }

    private string ToggleAdvancedConnectionLogPage()
    {
      showAdvancedConnectionLogPage = !showAdvancedConnectionLogPage;
      connectionLogGroupBox.Visible = showAdvancedConnectionLogPage;
      return "";
    }

#endregion ConnectionLogPage

#region FilterSchemataPage

    private string InitializeFilterSchemataPage()
    {
      List<int> indexes= new List<int>();
      pluginBE.def_schemata_selection(ref indexes);
      foreach (int n in indexes)
        schemataListView.Items[n].Selected = true;
      schemataListView.Select();
      return "";
    }

    private string CleanupFilterSchemataPage()
    {
      return "";
    }

    private string ProcessFilterSchemataPage()
    {
      return "";
    }

    private void cbShowAsList_CheckedChanged(object sender, EventArgs e)
    {
      CheckBox cb = sender as CheckBox;
      if (null == cb)
        return;

      ListView.SelectedIndexCollection indices = schemataListView.SelectedIndices;
      schemataListView.View = cb.Checked ? View.List : View.LargeIcon;
      int imageIndex = cb.Checked ? schema_small_image_index : schema_large_image_index;
      foreach (ListViewItem item in schemataListView.Items)
        item.ImageIndex = imageIndex;
      foreach (int index in indices)
        schemataListView.Items[index].Selected = true;
      schemataListView.Select();
    }

#endregion FilterSchemataPage

#region SchemataObjectsExtractionLogPage

    private string InitializeSchemataObjectsExtractionLogPage()
    {
      List<String> messages = new List<String>();
      string errMsg = "";

      taskLblRetrObjInfo.TaskState = MySQLTaskStatusLabelState.TaskOpen;
      taskLblCheckRetrObjInfoResults.TaskState = MySQLTaskStatusLabelState.TaskOpen;
      StepProlog(lSyncResult, lSyncStage);

      TaskMediator(schemataObjectsExtractionLogPanel, schemataObjectsExtractionLogTextBox, messages);

      // next task
      if (0 == errMsg.Length)
      {
        // restrict objects retrieving by selected schemata
        IEnumerator enumerator = schemataListView.SelectedIndices.GetEnumerator();
        if (null == enumerator)
          taskLblRetrObjInfo.TaskState = MySQLTaskStatusLabelState.TaskCompleted;
        else
        {
          lSyncStage.Text = "Retrieving objects description from selected schemata...";
          schemataObjectsExtractionLogPanel.Update();

          List<int> schemataIndexes = new List<int>(schemataListView.SelectedIndices.Count);
          while (enumerator.MoveNext())
            schemataIndexes.Add((int)enumerator.Current);
          pluginBE.schemata_selection(schemataIndexes, false);

          try
          {
            pluginBE.load_db_objects(Db_object_type.dbotTable);
            pluginBE.load_db_objects(Db_object_type.dbotView);
            pluginBE.load_db_objects(Db_object_type.dbotRoutine);
            pluginBE.load_db_objects(Db_object_type.dbotTrigger);
              //!            || !pluginBE.load_db_objects(Db_object_type.dbotUser, ref messages)
            taskLblRetrObjInfo.TaskState = MySQLTaskStatusLabelState.TaskCompleted;
          }
          catch (Exception)
          {
            errMsg = "Failed to fetch selected schemata objects.";
            taskLblRetrObjInfo.TaskState = MySQLTaskStatusLabelState.TaskError;
          }
        }
      }
      else
        taskLblRetrObjInfo.TaskState = MySQLTaskStatusLabelState.TaskCompleted;

      TaskMediator(schemataObjectsExtractionLogPanel, schemataObjectsExtractionLogTextBox, messages);

      // next task
      if (0 == errMsg.Length)
      {
        lSyncStage.Text = "Checking results...";
        schemataObjectsExtractionLogPanel.Update();

        taskLblCheckRetrObjInfoResults.TaskState = MySQLTaskStatusLabelState.TaskCompleted;
      }
      else
        taskLblCheckRetrObjInfoResults.TaskState = MySQLTaskStatusLabelState.TaskError;

      messages.Add(errMsg);
      TaskMediator(schemataObjectsExtractionLogPanel, schemataObjectsExtractionLogTextBox, messages);

      StepEpilog(errMsg, lSyncResult, lSyncStage);

      //return errMsg;
      return "";
    }

    private string CleanupSchemataObjectsExtractionLogPage()
    {
      schemataObjectsExtractionLogTextBox.Text = "";
      return "";
    }

    private string ProcessSchemataObjectsExtractionLogPage()
    {
      return "";
    }

    private string ToggleAdvancedSchemataObjectsExtractionLogPage()
    {
      showAdvancedSchemataObjectsExtractionLogPage = !showAdvancedSchemataObjectsExtractionLogPage;
      schemataObjectsExtractionLogGroupBox.Visible = showAdvancedSchemataObjectsExtractionLogPage;
      return "";
    }

#endregion SchemataObjectsExtractionLogPage

#region Filter Page Routines

    private void ReCreateFilter(ref SchemaObjectFilter filter, Db_object_type db_object_type)
    {
      if (pluginBE.db_objects_selection_model(db_object_type).total_items_count() > 0)
      {
        filter = master_schema_obj_filter_set.AddFilter();

        dbObjectFilters[db_object_type] = filter;

        filter.SetObjectType(pluginBE.db_objects_struct_name_by_type(db_object_type));
        filter.SetupModels(
          pluginBE.db_objects_selection_model(db_object_type),
          pluginBE.db_objects_exclusion_model(db_object_type));
        filter.RefreshModels();
      }
      else
        filter = null;
    }

    private string InitFilterPage()
    {
      filterPanel.SuspendLayout();

      if (master_schema_obj_filter_set == null)
      {
        master_schema_obj_filter_set = new MasterSchemaObjectFilterSet(wizard.GrtManager);
        master_schema_obj_filter_set.Parent = filterPanel;
        master_schema_obj_filter_set.Dock = DockStyle.Fill;
      }
      else
        master_schema_obj_filter_set.Reset();

      ReCreateFilter(ref trigger_filter, Db_object_type.dbotTrigger);
      ReCreateFilter(ref routine_filter, Db_object_type.dbotRoutine);
      ReCreateFilter(ref view_filter, Db_object_type.dbotView);
      ReCreateFilter(ref table_filter, Db_object_type.dbotTable);
      ReCreateFilter(ref user_filter, Db_object_type.dbotUser);

      filterPanel.ResumeLayout();

      return "";
    }

    private string CleanupFilterPage()
    {
      return "";
    }

    private string ProcessFilterPage()
    {
      // restrict objects importing by selected types
      foreach (Db_object_type db_object_type in dbObjectFilters.Keys)
        pluginBE.db_objects_activated(db_object_type, dbObjectFilters[db_object_type].TypeActivated);
      return "";
    }

    private string IsFilterToBeSkipped()
    {
      return (schemataListView.SelectedItems.Count == 0) ? "skip" : "";
    }

#endregion


#region Diff Tree Page routines

    private string InitTreePage()
    {
      if(diffTreeControl != null)
        diffTreeControl.Parent = null;  // release old control

      diffTreeControl = new DiffTreeControl(
        basePluginBE, schemataNames, pluginBE.db_catalog(), new MySQL.Grt.GrtValue());
      diffTreeControl.Parent = treePanel;
      diffTreeControl.Dock = DockStyle.Fill;
      diffTreeControl.Padding = new System.Windows.Forms.Padding(5);
      diffTreeControl.Margin = new System.Windows.Forms.Padding(5);
      return "";
    }

    private string CleanupTreePage()
    {
      return "";
    }

    private string ProcessTreePage()
    {
      outputScriptTextBox.Text = diffTreeControl.generate_diff_tree_script();
      return "";
    }

#endregion

#region Generated Script Page

    private string ProcessGeneratedScriptPage()
    {
      return "";
    }

#endregion

#region ExecScriptLogPage
    
    private int ProcessSqlScriptMsg(int msgType, String message)
    {
      if ((Msg_type)msgType == Msg_type.MT_error)
        syncScriptSucceeded = false;
      execScriptLogTextBox.Text += message;
      execScriptLogTextBox.Update();
      return 0;
    }

    private int ProcessSqlScriptProgress(float progressState, String message)
    {
      pbExecScript.Value = (int)(progressState * (float)100);
      pbExecScript.Update();
      return 0;
    }

    private int ProcessSqlScriptFail(String message)
    {
      execScriptLogTextBox.Text += message;
      execScriptLogTextBox.Update();

      taskLblExecScript.TaskState = MySQLTaskStatusLabelState.TaskError;
      taskLblCheckSyncResults.TaskState = MySQLTaskStatusLabelState.TaskError;

      StepEpilog(message, lExecScriptResult, lExecScriptStage);
      wizard.RestoreControlsState(message);
      return 0;
    }

    private int ProcessSqlScriptFinish()
    {
      MySQLTaskStatusLabelState taskStatus = syncScriptSucceeded ? MySQLTaskStatusLabelState.TaskCompleted : MySQLTaskStatusLabelState.TaskError;
      taskLblExecScript.TaskState = taskStatus;
      taskLblCheckSyncResults.TaskState = taskStatus;
      String errMsg = (syncScriptSucceeded ? "" : "At least one statement from script failed.");
      StepEpilog(errMsg, lExecScriptResult, lExecScriptStage);
      wizard.RestoreControlsState("");

      if (syncScriptSucceeded)
        diffTreeControl.update_model_old_names();

      return 0;
    }

    private string InitializeExecScriptLogPage()
    {
      basePluginBE.apply_changes_to_model();

      List<String> messages = new List<String>();
      string errMsg = "";

      taskLblExecScript.TaskState = MySQLTaskStatusLabelState.TaskOpen;
      taskLblCheckSyncResults.TaskState = MySQLTaskStatusLabelState.TaskOpen;
      StepProlog(lExecScriptResult, lExecScriptStage);

      lExecScriptDetail.Text = "";
      pbExecScript.Minimum = 0;
      pbExecScript.Maximum = 100;

      TaskMediator(execScriptLogPanel, execScriptLogTextBox, messages);

      // initialization task
      syncScriptSucceeded = true;
      pluginBE.task_msg_cb(ProcessSqlScriptMsg);
      pluginBE.task_progress_cb(ProcessSqlScriptProgress);
      pluginBE.task_fail_cb(ProcessSqlScriptFail);
      pluginBE.task_finish_cb(ProcessSqlScriptFinish);

      wizard.SkipRestoreControlsStateOnce();

      TaskMediator(execScriptLogPanel, execScriptLogTextBox, messages);

      // next task
      if (0 == errMsg.Length)
      {
        lExecScriptStage.Text = "Synchronizing selected objects...";
        execScriptLogPanel.Update();

        pluginBE.set_option("ScriptToApply", outputScriptTextBox.Text);
        //pluginBE.exec_task();
        pluginBE.start_apply_script_to_db();
        
        /* //!
        if (pluginBE.start_apply_script_to_db())
        {
          errMsg = "Error while synchronizing db objects.";
          taskLblExecScript.TaskState = MySQLTaskStatusLabelState.TaskError;
        }
        else
          taskLblExecScript.TaskState = MySQLTaskStatusLabelState.TaskCompleted;
        //*/
      }
      else
        taskLblExecScript.TaskState = MySQLTaskStatusLabelState.TaskError;

      TaskMediator(execScriptLogPanel, execScriptLogTextBox, messages);
/*
      // next task
      if (0 == errMsg.Length)
      {
        lExecScriptStage.Text = "Checking results...";
        execScriptLogPanel.Update();

        taskLblCheckSyncResults.TaskState = MySQLTaskStatusLabelState.TaskCompleted;
      }
      else
        taskLblCheckSyncResults.TaskState = MySQLTaskStatusLabelState.TaskError;

      TaskMediator(execScriptLogPanel, execScriptLogTextBox, messages);

      StepEpilog(errMsg, lExecScriptResult, lExecScriptStage);
*/
      return errMsg;
    }

    private string CleanupExecScriptLogPage()
    {
      execScriptLogTextBox.Text = "";
      return "";
    }

    private string ProcessExecScriptLogPage()
    {
      return "";
    }

    private string ToggleAdvancedExecScriptLogPage()
    {
      showAdvancedExecScriptLogPage = !showAdvancedExecScriptLogPage;
      importGroupBox.Visible = showAdvancedExecScriptLogPage;
      return "";
    }

    void ExecScriptFinished()
    {
      wizard.RestoreControlsState("");
    }

#endregion ExecScriptLogPage
  }
}