using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using MySQL.Grt;
using MySQL.Grt.Db;
using MySQL.Utilities;
using MySQL.GUI.Workbench.Plugins;

namespace MySQL.GUI.Workbench.Plugins.Db
{
  public partial class MasterSchemaObjectFilterSet : UserControl
  {
    protected GrtManager grtManager;
    protected DBObjectMasterFilterBE masterFilterBE;
    protected List<SchemaObjectFilter> filters;
    protected int bottom;

    public MasterSchemaObjectFilterSet()
    {
      InitializeComponent();
    }

    public MasterSchemaObjectFilterSet(GrtManager GrtManager)
      : this()
    {
      grtManager = GrtManager;
      masterFilterBE = new DBObjectMasterFilterBE(grtManager);
      filters = new List<SchemaObjectFilter>();
      ReloadStoredFilterSetList("");
    }

    public SchemaObjectFilter AddFilter()
    {
      SchemaObjectFilter filter = new SchemaObjectFilter(grtManager);

      filter.Parent = filtersPanel;
      filter.Dock = DockStyle.Top;
      filter.Margin = new Padding(5);

      filter.StoredFilterSetControl.SelectionChangeCommitted += new EventHandler(StoredFilterSetControl_SelectionChangeCommitted);

      masterFilterBE.add_filter(filter.Backend);
      filters.Add(filter);

      return filter;
    }

    public void Reset()
    {
      masterFilterBE.remove_all_filters();
      foreach (SchemaObjectFilter filter in filters)
      {
        filter.ResetModels();
        filter.Parent = null;
      }
      filters.Clear();
      cbStoredFilterSet.SelectedIndex = -1;
    }

    private void ReloadStoredFilterSetList(String storedFilterSetName)
    {
      cbStoredFilterSet.Items.Clear();
      List<String> names = new List<String>();
      masterFilterBE.load_stored_filter_set_list(ref names);
      foreach (String name in names)
      {
        cbStoredFilterSet.Items.Add(name);
        if (0 == storedFilterSetName.CompareTo(name))
          cbStoredFilterSet.SelectedIndex = cbStoredFilterSet.Items.Count - 1;
      }
    }

    private void StoredFilterSetControl_SelectionChangeCommitted(Object sender, EventArgs e)
    {
      cbStoredFilterSet.SelectedIndex = -1;
    }

    private void btnAddStoredFilterSet_Click(object sender, EventArgs e)
    {
      string storedFilterSetName;
      if (StringInputForm.ShowModal("Adding new stored master filter set", "", "Master Filter Set Name:", out storedFilterSetName) == DialogResult.OK)
      {
        if (storedFilterSetName.Length > 0)
        {
          List<String> filterSetNames= new List<String>();
          foreach (SchemaObjectFilter filter in filters)
          {
            if (0 < filter.Backend.filter_model().count())
            {
              string filterSetName = storedFilterSetName + " - " + filter.Backend.get_full_type_name();
              filter.AddStoredFilterSet(filterSetName);
              filterSetNames.Add(filterSetName);
            }
          }

          masterFilterBE.add_stored_filter_set(storedFilterSetName, filterSetNames);
          ReloadStoredFilterSetList(storedFilterSetName);
        }
      }
    }

    private void btnRemoveStoredFilterSet_Click(object sender, EventArgs e)
    {
      if (cbStoredFilterSet.Text.Length != 0)
        masterFilterBE.remove_stored_filter_set(cbStoredFilterSet.SelectedIndex);
      ReloadStoredFilterSetList("");
    }

    private void cbStoredFilterSet_SelectionChangeCommitted(object sender, EventArgs e)
    {
      List<int> indexes = new List<int>();
      masterFilterBE.load_stored_filter_set(cbStoredFilterSet.SelectedIndex, ref indexes);

      for (int n= 0, count= filters.Count; n < count; ++n)
        filters[n].StoredFilterSetIndex = indexes[n];
    }
  }
}
