using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI.Docking;
using Aga.Controls.Tree;
using MySQL.Grt;
using MySQL.Utilities;
using ScintillaNet;

namespace MySQL.GUI.Workbench.Plugins
{
  public partial class StoredNoteEditor : ObjectEditorPlugin
  {
    #region Member Variables

    private MySQL.Grt.StoredNoteEditorBE storedNoteEditorBE { get { return editorBE as MySQL.Grt.StoredNoteEditorBE; } }

    #endregion

    #region Constructors

    protected StoredNoteEditor()
    {
      InitializeComponent();
    }

    public StoredNoteEditor(GrtManager GrtManager, GrtValue GrtObject)
      : base(new MySQL.Grt.StoredNoteEditorBE(GrtManager, GrtObject))
    {
      InitializeComponent();

      // Init BE
      InitializingControls = true;
      try
      {
        RefreshFormData();
      }
      finally
      {
        InitializingControls = false;
      }

      TabText = storedNoteEditorBE.get_name();
    }

    #endregion

    #region Form implemenation

    protected override void RefreshFormData()
    {
      if (storedNoteEditorBE.is_sql_script())
      {
        textBox.ConfigurationManager.Language = "mysql";
        textBox.Font = grtManager.get_font_option("workbench.general.Editor:Font");
        textBox.Margins.Margin0.Width = 35;
      }
      else 
      {
        textBox.ConfigurationManager.Language = "";
        textBox.Font = new Font("Microsoft Sans Serif", 8.25f);
        textBox.Margins.Margin0.Width = 0;
      }

      bool isutf8;
      textBox.Text = storedNoteEditorBE.get_text(out isutf8);

      if (isutf8)
        textBox.Enabled = true;
      else
      {
        textBox.Enabled = false;
        textBox.Text= "The contents of this file cannot be displayed because it is in an unknown encoding.\r\n"+
        "You may save the file and view it with an external program.";
      }

      applyButton.Enabled = false;
      discardButton.Enabled = false;
    }

    private void applyButton_Click(object sender, EventArgs e)
    {
      storedNoteEditorBE.set_text(textBox.Text);
      RefreshFormData();
    }

    private void discardButton_Click(object sender, EventArgs e)
    {
      RefreshFormData();
    }

    private void saveButton_Click(object sender, EventArgs e)
    {
      if (saveFileDialog1.ShowDialog() == DialogResult.OK)
      {
        storedNoteEditorBE.save_file(saveFileDialog1.FileName);
      }
    }

    private void loadButton_Click(object sender, EventArgs e)
    {
      if (openFileDialog1.ShowDialog() == DialogResult.OK)
      {
        storedNoteEditorBE.load_file(openFileDialog1.FileName);
        RefreshFormData();
        applyButton.Enabled = true;
        discardButton.Enabled = true;
      }
    }

    private void cutItem_Click(object sender, EventArgs e)
    {
      textBox.Clipboard.Cut();
    }

    private void copyItem_Click(object sender, EventArgs e)
    {
      textBox.Clipboard.Copy();
    }

    private void pasteItem_Click(object sender, EventArgs e)
    {
      textBox.Clipboard.Paste();
    }

    private void selectAllItem_Click(object sender, EventArgs e)
    {
      textBox.Selection.SelectAll();
    }

    private void undoItem_Click(object sender, EventArgs e)
    {
      textBox.UndoRedo.Undo();
    }

    private void redoItem_Click(object sender, EventArgs e)
    {
      textBox.UndoRedo.Redo();
    }

    private void editorContextMenu_Opening(object sender, CancelEventArgs e)
    {
      // Enable/Disable menu items depending on the edit content.
      undoItem.Enabled = textBox.UndoRedo.IsUndoEnabled;
      cutItem.Enabled = textBox.Selection.Length > 0;
      copyItem.Enabled = textBox.Selection.Length > 0;
      pasteItem.Enabled = textBox.Clipboard.CanPaste;
      selectAllItem.Enabled = textBox.TextLength > 0;
    }

    private const uint TEXT_MODIFIED_FLAGS = Constants.SC_MOD_BEFOREDELETE | Constants.SC_MOD_BEFOREINSERT |
      Constants.SC_MOD_DELETETEXT | Constants.SC_MOD_INSERTTEXT;

    private void textBox_DocumentChange(object sender, ScintillaNet.NativeScintillaEventArgs e)
    {
      SCNotification scn = e.SCNotification;
      int modType = scn.modificationType;

      if ((modType & TEXT_MODIFIED_FLAGS) > 0)
      {
        applyButton.Enabled = true;
        discardButton.Enabled = true;
      }
    }

    #endregion

  }
}