#!/usr/bin/python

import os
import string
import time


def locateCodeFiles(path, ignoreList):
    files= []
    for f in os.listdir(path):
        if f in ignoreList or os.path.join(path, f) in ignoreList:
            continue
        if os.path.isdir(os.path.join(path, f)):
            files+= locateCodeFiles(os.path.join(path, f), ignoreList)
        elif os.path.splitext(f)[1] in [".c", ".cc", ".cpp"]:
            files.append(os.path.join(path, f))
    return files


def getCovStats(file):
    lines= open(file, "r").readlines()

    unexecutedLines= []
    notExecuted= 0
    executed= 0

    for line in lines:
        line= line.strip()
        if not line:
            continue
        parts= line.split(":", 2)
        execCount= parts[0].strip()
        lineNum= parts[1].strip()
        code= parts[2].strip()

        if execCount == "-":
            continue
        if execCount[0] == "#":
            notExecuted+= 1
            unexecutedLines.append(int(lineNum))
        elif execCount[0] in string.digits:
            executed+= 1
        else:
            assert 0

    return executed, notExecuted, unexecutedLines


def checkTestedFiles(pathList):
    tested= []
    notTestedL= pathList[:]
    for f in pathList:
        file= os.path.basename(f)
        if os.path.exists(file+".gcov"):
            stats= getCovStats(file+".gcov")

            notTestedL.remove(f)
            tested.append((f, stats))
            
    return tested, notTestedL




ignoreFiles=[".svn", ".deps", ".libs", "test", "ftgl", "tests", "newt"]


files= locateCodeFiles("..", ignoreFiles)
tested, notTested= checkTestedFiles(files)
print tested
total= 0
covered= 0
notCovered= 0
fileStats= {}
for f, stats in tested:
    fileStats[f]= {
    "covered":stats[0],
    "notCovered":stats[1],
    "tested":stats[0]+stats[1],
    "notCoveredLineNumbers":stats[2]
    }
    covered+= fileStats[f]["covered"]
    notCovered+= fileStats[f]["notCovered"]
    total+= fileStats[f]["tested"]

stats= {
"fileCount":len(files),
"testedFileCount": len(tested),
"testedCoveredLines": covered,
"testedNotCoveredLines": notCovered,
"testedTotalLines": total,
"tested_pct":float(len(tested))*100/len(files),
"coverage_pct":float(covered)*100/total,
}


template= {"files":None, "fileCount":0, "testedFileCount":0, "covered":0, "notCovered":0, "tested":0}
libraryStats= {}
for f in files:
    parts= f.split("/")
    library= parts[1]
    lstats= libraryStats.get(library, template.copy())
    lstats["fileCount"]+= 1
    if not lstats["files"]:
        lstats["files"]= [f]
    else:
        lstats["files"].append(f)
    if fileStats.has_key(f):
        lstats["testedFileCount"]+= 1
        lstats["covered"]+= fileStats[f]["covered"]
        lstats["notCovered"]+= fileStats[f]["notCovered"]
        lstats["tested"]+= fileStats[f]["tested"]
    else:
        pass
    libraryStats[library]= lstats


print "<html><title>Test Coverage Report: %s</title><body>"%time.strftime("%Y/%m/%d")
print "<h1>Test Coverage Report For %s</h1>"%time.strftime("%Y/%m/%d")

print """
<p>Total Tested Source Files: %(testedFileCount)s of %(fileCount)s files  (%(tested_pct).2f%%)
<p>Coverage in Tested Files: %(testedCoveredLines)s of %(testedTotalLines)s lines (%(coverage_pct).2f%%)

<p><small><em>Coverage in Tested Files means the number of lines that were actually executed in tests,
in relation to the total number of lines in <em>files that were tested at all</em>.
Files that were not handled in tests, are not counted.</em></small>


""" % stats

print "<h1>Coverage by Library</h1>"
for lib, lstats in libraryStats.items():
    
    if lstats["tested"] > 0:
        covered_pct= float(lstats["covered"])*100/lstats["tested"]
    else:
        covered_pct= 0
    filecov_pct= float(lstats["testedFileCount"]*100)/lstats["fileCount"]
    s= lstats.copy()
    s.update({"covered_pct":covered_pct, "filecov_pct":filecov_pct})
    print "<h2><a href='#%s'>%s</a></h2>"%(lib,lib)
    print """<p>Number of Files Tested: %(testedFileCount)s of %(fileCount)s files    <b>(%(filecov_pct).2f%%)</b>
<p>Coverage in Tested Files: %(covered)s of %(tested)s lines   (%(covered_pct).2f%%)
"""%s



print "<h1>Details by File</h1>"
for lib, lstats in libraryStats.items():
    print "<a name='%s'><h2>%s</h2></a>"%(lib,lib)

    print "<table border=0 width=100%>"
    for f in lstats["files"]:
        print "<tr>"
        print "<td>%s</td>"%f
        fs= fileStats.get(f)
        if fs is None or fs["tested"] == 0:
            print "<td width=100%%>Not Tested</td>"
        else:
            print "<td width=100%%>%(covered)s of %(tested)s lines</td>" % fs
        print "</tr>"
    print "</table>"

print "</body></html>"
