
#include <grtpp.h>
#include "structs.test.h"

using namespace grt;



inline BaseListRef create_list_with_varied_data(GRT *grt)
{
  BaseListRef l(grt);

  // 1
  l.ginsert(IntegerRef(111));

  // 2
  l.ginsert(IntegerRef(222));
  
  // 3
  l.ginsert(StringRef("test"));
  
  // 4
  l.ginsert(DoubleRef(123.456));
  
  // 5
  l.ginsert(StringRef("test"));

  // 6
  IntegerListRef il(grt);

  for (int i= 0; i < 5; i++)
    il.insert(i);
  l.ginsert(il);

  // 7
  DictRef d(grt);

  d.set("item1", IntegerRef(1));
  d.set("item2", DoubleRef(2.2));
  d.set("item3", StringRef("test"));
  l.ginsert(d);

  // 8
  l.ginsert(StringRef("test"));
  
  // 9
  l.ginsert(StringRef("test"));
  
  // 10
  l.ginsert(StringRef("test"));

  return l;
}


inline StringListRef create_string_list(GRT *grt, int size)
{
  StringListRef list(grt);
  
  for (int i= 0; i < size; i++)
  {
    char buffer[10];
    sprintf(buffer, "test%i", i+1);
    list.insert(StringRef(buffer));
  }

  return list;
}


inline IntegerListRef create_int_list(GRT *grt, int size)
{
  IntegerListRef list(grt);
  
  for (int i= 0; i < size; i++)
  {
    list.insert(i);
  }
  return list;
}



inline DictRef create_dict_with_varied_data(GRT *grt)
{
  DictRef dict(grt);

  // out of alphabetical order on purpose
  dict.set("k4", IntegerRef(444));
  dict.set("k1", IntegerRef(111));
  dict.set("k5", StringRef("test5"));
  dict.set("k3", StringRef("test3"));
  dict.set("k2", DoubleRef(123.456));
  
  IntegerListRef list(grt);
  
  for (int i= 1; i <= 5; i++)
    list.insert(IntegerRef(i));

  dict.set("k6", list);

  return dict;
}



inline DictRef create_dict_with_int_data(GRT *grt)
{
  DictRef dict(grt, IntegerType);

  // out of alphabetical order on purpose
  for (int i= 0; i < 9; i++)
  {
    char buffer[10];
    
    sprintf(buffer, "k%i", i+1);
    
    dict.set(buffer, IntegerRef(i+1));
  }

  return dict;
}




inline ValueRef create_grt_tree1(GRT *grt)
{
  DictRef root(grt);
  
  ListRef<test_Book> book_list(grt);

  test_BookRef book1(grt);
  book1->title("Some Book");
  book_list.insert(book1);

  test_BookRef book2(grt);
  book2->title("The Illiad");
  book_list.insert(book2);

  root.set("books", book_list);



  root.set("somelist", create_list_with_varied_data(grt));

  
  root.set("somedict", create_dict_with_varied_data(grt));
  
  return root;
}
