#ifndef _TEST_H_
#define _TEST_H_

#ifndef _WIN32
#include <string.h>
#include <string>
#include <algorithm>
#endif

#include "tcp_helpers.h"
#include "tut.h"
#include "test_helpers.h"
#include "tut_helpers.h"

#define __TOSTR(c) ___TOSTR(c)
#define ___TOSTR(c) #c

#define BEGIN_TEST_DATA_CLASS(module)   \
   namespace tut {                      \
   class module;                        \
   template<> class Test_object_base<module> {  \

#define TEST_DATA_CONSTRUCTOR(module)   \
  Test_object_base<module>()

#define TEST_DATA_DESTRUCTOR(module)   \
  ~Test_object_base<module>()

#define END_TEST_DATA_CLASS };}

#define TEST_MODULE(module, moddesc)  \
  namespace tut {\
  class module {}; \
  typedef Test_group<module>::object Test_object; \
  static Test_group<module> test_group_singleton(&test_params, __TOSTR(module), moddesc);

#define DISABLED_TEST_MODULE(module, moddesc)  \
  namespace tut {\
  class module {}; \
  typedef Test_group<module>::object Test_object; \
  static Test_group<void> test_group_singleton(&test_params, __TOSTR(module)" [DISABLED]", moddesc);

#define TEST_FUNCTION(number) \
  template<>  \
  template<>  \
  void Test_object::test<number>()

#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
#define DIR_DELIMITER  '\\'
#else
#define DIR_DELIMITER  '/'
#endif

#define __LOCATION (strrchr(__FILE__":"__TOSTR(__LINE__), DIR_DELIMITER)+1)

#define ensure(text, test) ensure(std::string(__LOCATION).append(": ").append(text).c_str(), (test))
#define ensure_equals(text, actual, expected) ensure_equals(std::string(__LOCATION).append(": ").append(text).c_str(), (actual), (expected))

#define ensure_png_equals(text, file) ensure(std::string(__LOCATION).append(": ").append(text).c_str(), test_group_singleton.compare_images(file, std::string("refdata/")+file))

// this macros set does not require message and does not provide advanced comparing capabilities
#define assure(expr) ensure(#expr,(expr))
#define assure_equal(expr1,expr2) ensure_equals(#expr1"=="#expr2, (expr1) , (expr2))
#define assure_not_equal(expr1,expr2) ensure(#expr1"!="#expr2, (expr1) != (expr2))
#define assure_not_null(expr1) ensure(#expr1" not null", ((expr1) != NULL))
#define assure_null(expr1)    ensure(#expr1" null", ((expr1) == NULL))
#define assure_grt_values_equal(grt1,grt2)    \
  grt_ensure_equals(#grt1 " not the same as " #grt2, \
  ValueRef(grt1), ValueRef(grt2))


namespace tut
{
template<class _InIt1, class _InIt2> inline
void assure_containers_equal(_InIt1 _First1, _InIt1 _Last1,
    _InIt2 _First2, _InIt2 _Last2)
{
  ensure_equals("Containers have different sizes.", std::distance(_First1, _Last1), std::distance(_First2, _Last2));

  _InIt1 iter1= _First1;
  _InIt2 iter2= _First2;
  for(; iter1 != _Last1; iter1++, iter2++)
    assure_equal(*iter1, *iter2);
}

template<class _InIt1, class _InIt2, class _Op> inline
void assure_containers_equal(_InIt1 _First1, _InIt1 _Last1,
    _InIt2 _First2, _InIt2 _Last2, _Op Op)
{
  ensure_equals("Containers have different sizes.", std::distance(_First1, _Last1), std::distance(_First2, _Last2));

  _InIt1 iter1= _First1;
  _InIt2 iter2= _First2;
  for(; iter1 != _Last1; iter1++, iter2++)
    assure(Op(iter1, iter2));
}

#ifndef UPPER_BOUND
#define UPPER_BOUND(x) ((x==NULL)? 0: sizeof(x)/sizeof(x[0]))
#endif

}

extern Global_test_parameters *test_params;


// these are needed because g++ complains something about stuff being
// defined in different namespace than declaration
#define END_TESTS }


#endif // _TEST_H_

