#include "stdafx.h"

#include "test.h"
#include "grt/common.h"

using namespace bec;

BEGIN_TEST_DATA_CLASS(test_utilities_test)
END_TEST_DATA_CLASS;

TEST_MODULE(test_utilities_test, "Test utilities testing");

TEST_FUNCTION(1)
{
  ensure_equals("fmt", fmt("%s", "hello"), "hello");
  ensure_equals("fmt", fmt(""), "");
  ensure_equals("fmt i", fmt("%s %i", "hello", 32), "hello 32");
  ensure_equals("fmt", fmt("hihi"), "hihi");
}

TEST_FUNCTION(2)
{
  ensure_equals("replace_string", replace_string("hello$", "$", " world"),
                "hello world");
  ensure_equals("replace_string", replace_string("", "$", "!!"), "");
  ensure_equals("replace_string", replace_string("$", "$", "!!"), "!!");
  ensure_equals("replace_string", replace_string("hello", "$", "!!"), "hello");
  ensure_equals("replace_string", replace_string("$hello", "$", "!!"), "!!hello");
  ensure_equals("replace_string", replace_string("hello$", "$", "!!"), "hello!!");
  ensure_equals("replace_string", replace_string("$hello$", "$", "!!"), "!!hello!!");
  ensure_equals("replace_string", replace_string("$$hello", "$", "!!"), "!!!!hello");
  ensure_equals("replace_string", replace_string("shello", "el", "!!"), "sh!!lo");
  ensure_equals("replace_string", replace_string("ab$d", "$", "c$"), "abc$d");
}

TEST_FUNCTION(3)
{
  std::vector<std::string> res;
  
  res= split_string("12345", ";");
  
  ensure_equals("split_string", res.size(), 1U);
  ensure_equals("split_string", res[0], "12345");
  
  res= split_string("12;45", ";");
  ensure_equals("split_string", res.size(), 2U);
  ensure_equals("split_string", res[0], "12");
  ensure_equals("split_string", res[1], "45");

  res= split_string("12;45;", ";");
  ensure_equals("split_string", res.size(), 3U);
  ensure_equals("split_string", res[0], "12");
  ensure_equals("split_string", res[1], "45");
  ensure_equals("split_string", res[2], "");

  res= split_string(";12 45", ";");
  ensure_equals("split_string", res.size(), 2U);
  ensure_equals("split_string", res[0], "");
  ensure_equals("split_string", res[1], "12 45");
  
  res= split_string("$$12$$45$$", "$$");
  ensure_equals("split_string", res.size(), 4U);
  ensure_equals("split_string", res[0], "");
  ensure_equals("split_string", res[1], "12");
  ensure_equals("split_string", res[2], "45");
  ensure_equals("split_string", res[3], "");

  res= split_string("", "#");
  ensure_equals("split_string", res.size(), 0U);
}

TEST_FUNCTION(4)
{
  ensure_equals("escape_sql_string", escape_sql_string("hello"), "hello");
  ensure_equals("escape_sql_string", escape_sql_string("he\nllo"), "he\\nllo");
  ensure_equals("escape_sql_string", escape_sql_string("\nhello\r"), "\\nhello\\r");
  ensure_equals("escape_sql_string", escape_sql_string("'hello'"), "\\'hello\\'");
  ensure_equals("escape_sql_string", escape_sql_string("\"hello\""), "\\\"hello\\\"");
  ensure_equals("escape_sql_string", escape_sql_string("\'hello\"X"), "\\'hello\\\"X");
  ensure_equals("escape_sql_string", escape_sql_string("'\"\"'"), "\\'\\\"\\\"\\'");
}

END_TESTS
